import collections
from abjad import enums
from abjad.system.AbjadObject import AbjadObject


class Iteration(AbjadObject):
    r"""
    Iteration.

    ..  container:: example

        Iterates leaves:

        ..  container:: example

            >>> staff = abjad.Staff("c'4 e'4 d'4 f'4")
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    e'4
                    d'4
                    f'4
                }

        ..  container:: example

            >>> for leaf in abjad.iterate(staff).leaves():
            ...     leaf
            Note("c'4")
            Note("e'4")
            Note("d'4")
            Note("f'4")

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Collaborators'

    __slots__ = (
        '_client',
        )

    ### INITIALIZER ###

    def __init__(self, client=None):
        assert not isinstance(client, str), repr(client)
        self._client = client

    ### PRIVATE METHODS ###

    def _depth_first(
        self,
        capped=True,
        direction=None,
        forbid=None,
        unique=True,
        ):
        r"""
        Iterates depth first.

        ..  container:: example

            Iterates depth first:

            ..  container:: example

                >>> score = abjad.Score([])
                >>> score.append(abjad.Staff("c''4 ~ c''8 d''8 r4 ef''4"))
                >>> score.append(abjad.Staff("r8 g'4. ~ g'8 r16 f'8. ~ f'8"))
                >>> abjad.show(score) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(score)
                    \new Score
                    <<
                        \new Staff
                        {
                            c''4
                            ~
                            c''8
                            d''8
                            r4
                            ef''4
                        }
                        \new Staff
                        {
                            r8
                            g'4.
                            ~
                            g'8
                            r16
                            f'8.
                            ~
                            f'8
                        }
                    >>

            ..  container:: example

                >>> for component in abjad.iterate(score)._depth_first():
                ...     component
                ...
                <Score<<2>>>
                Staff("c''4 ~ c''8 d''8 r4 ef''4")
                Note("c''4")
                Note("c''8")
                Note("d''8")
                Rest('r4')
                Note("ef''4")
                Staff("r8 g'4. ~ g'8 r16 f'8. ~ f'8")
                Rest('r8')
                Note("g'4.")
                Note("g'8")
                Rest('r16')
                Note("f'8.")
                Note("f'8")

        ..  container:: example

            Iterates depth first in reverse:

            ..  container:: example

                >>> score = abjad.Score([])
                >>> score.append(abjad.Staff("c''4 ~ c''8 d''8 r4 ef''4"))
                >>> score.append(abjad.Staff("r8 g'4. ~ g'8 r16 f'8. ~ f'8"))
                >>> abjad.show(score) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(score)
                    \new Score
                    <<
                        \new Staff
                        {
                            c''4
                            ~
                            c''8
                            d''8
                            r4
                            ef''4
                        }
                        \new Staff
                        {
                            r8
                            g'4.
                            ~
                            g'8
                            r16
                            f'8.
                            ~
                            f'8
                        }
                    >>

            ..  container:: example

                >>> agent = abjad.iterate(score)
                >>> for component in agent._depth_first(direction=abjad.Right):
                ...     component
                ...
                <Score<<2>>>
                Staff("r8 g'4. ~ g'8 r16 f'8. ~ f'8")
                Note("f'8")
                Note("f'8.")
                Rest('r16')
                Note("g'8")
                Note("g'4.")
                Rest('r8')
                Staff("c''4 ~ c''8 d''8 r4 ef''4")
                Note("ef''4")
                Rest('r4')
                Note("d''8")
                Note("c''8")
                Note("c''4")

        ..  container:: example

            Iterates depth first with grace notes:

            ..  container:: example

                >>> voice = abjad.Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> container = abjad.GraceContainer("cf''16 bf'16")
                >>> abjad.attach(container, voice[1])
                >>> container = abjad.AfterGraceContainer("af'16 gf'16")
                >>> abjad.attach(container, voice[1])
                >>> abjad.show(voice) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(voice)
                    \new Voice
                    {
                        c'8
                        [
                        \grace {
                            cf''16
                            bf'16
                        }
                        \afterGrace
                        d'8
                        {
                            af'16
                            gf'16
                        }
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for component in abjad.iterate(voice)._depth_first():
                ...     component
                ...
                Voice("c'8 d'8 e'8 f'8")
                Note("c'8")
                Note("d'8")
                GraceContainer("cf''16 bf'16")
                Note("cf''16")
                Note("bf'16")
                AfterGraceContainer("af'16 gf'16")
                Note("af'16")
                Note("gf'16")
                Note("e'8")
                Note("f'8")

        Returns generator.
        """
        import abjad
        if direction is None:
            direction = enums.Left
        def _next_node_depth_first(component, total):
            """
            If client has unvisited components, return next unvisited
            component in client.

            If client has no univisited components, return client's parent.
            """
            # if component is a container with not-yet-returned children
            if (isinstance(component, abjad.Container) and
                0 < len(component) and total < len(component)):
                # return next not-yet-returned child
                return component[total], 0
            # if component is a leaf with grace container attached
            elif getattr(component, '_grace_container', None) is not None:
                return component._grace_container, 0
            # if component is a leaf with after grace container attached
            elif (getattr(component, '_after_grace_container', None)
                is not None):
                return component._after_grace_container, 0
            # if component is grace container with all children returned
            elif hasattr(component, '_carrier'):
                carrier = component._carrier
                # if grace container has no carrier
                if carrier is None:
                    return None, None
                # if there's also an after grace container
                if (not isinstance(component, abjad.AfterGraceContainer) and
                    carrier._after_grace_container is not None):
                    return carrier._after_grace_container, 0
                carrier_parent = carrier._parent
                if carrier_parent is None:
                    return None, None
                # advance to next node in carrier parent
                return carrier_parent, carrier_parent.index(carrier) + 1
            else:
                parent = component._parent
                if parent is None:
                    return None, None
                return parent, parent.index(component) + 1
        def _previous_node_depth_first(component, total=0):
            """
            If client has unvisited components, return previous unvisited
            component in client.

            If client has no univisited components, return client's parent.
            """
            if (isinstance(component, abjad.Container) and
                0 < len(component) and total < len(component)):
                return component[len(component) - 1 - total], 0
            else:
                parent = component._parent
                if parent is not None:
                    return parent, len(parent) - parent.index(component)
                else:
                    return None, None
        def _handle_forbidden_node(node, queue):
            node_parent = node._parent
            if node_parent is not None:
                rank = node_parent.index(node) + 1
                node = node_parent
            else:
                node, rank = None, None
            queue.pop()
            return node, rank
        def _advance_node_depth_first(node, rank, direction):
            if direction is enums.Left:
                node, rank = _next_node_depth_first(node, rank)
            else:
                node, rank = _previous_node_depth_first(node, rank)
            return node, rank
        def _is_node_forbidden(node, forbid):
            if forbid is None:
                return False
            elif forbid == 'simultaneous':
                return getattr(node, 'is_simultaneous', False)
            else:
                return isinstance(node, forbid)
        def _find_yield(node, rank, queue, unique):
            if hasattr(node, 'components'):
                try:
                    visited = node is queue[-1]
                except IndexError:
                    visited = False
                if not visited or unique is not True:
                    queue.append(node)
                    return node
                elif rank == len(node):
                    queue.pop()
                    return None
            else:
                return node
        assert isinstance(self.client, abjad.Component)
        component = self.client
        client_parent, node, rank = component._parent, component, 0
        queue = collections.deque([])
        while node is not None and not (capped and node is client_parent):
            result = _find_yield(node, rank, queue, unique)
            if result is not None:
                yield result
            if _is_node_forbidden(node, forbid):
                node, rank = _handle_forbidden_node(node, queue)
            else:
                node, rank = _advance_node_depth_first(
                    node, rank, direction)
        queue.clear()

    def _logical_voice(self, prototype=None, reverse=False):
        r"""
        Iterates logical voice.

        ..  container:: example

            Iterates logical voice from first leaf in score:

            ..  container:: example

                >>> container_1 = abjad.Container([
                ...     abjad.Voice("c'8 d'8"),
                ...     abjad.Voice("e'8 f'8"),
                ...     ])
                >>> container_1.is_simultaneous = True
                >>> container_1[0].name = 'Voice 1'
                >>> abjad.override(container_1[0]).stem.direction = abjad.Down
                >>> container_1[1].name = 'Voice 2'
                >>> container_2 = abjad.Container([
                ...     abjad.Voice("g'8 a'8"),
                ...     abjad.Voice("b'8 c''8"),
                ...     ])
                >>> container_2.is_simultaneous = True
                >>> container_2[0].name = 'Voice 1'
                >>> abjad.override(container_2[0]).stem.direction = abjad.Down
                >>> container_2[1].name = 'Voice 2'
                >>> staff = abjad.Staff([container_1, container_2])
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        <<
                            \context Voice = "Voice 1"
                            \with
                            {
                                \override Stem.direction = #down
                            }
                            {
                                c'8
                                d'8
                            }
                            \context Voice = "Voice 2"
                            {
                                e'8
                                f'8
                            }
                        >>
                        <<
                            \context Voice = "Voice 1"
                            \with
                            {
                                \override Stem.direction = #down
                            }
                            {
                                g'8
                                a'8
                            }
                            \context Voice = "Voice 2"
                            {
                                b'8
                                c''8
                            }
                        >>
                    }

            ..  container:: example

                >>> selector = abjad.select().leaves()
                >>> leaves = selector(staff)
                >>> leaf = leaves[0]
                >>> for note in abjad.iterate(leaf)._logical_voice(
                ...     prototype=abjad.Note,
                ...     ):
                ...     note
                ...
                Note("c'8")
                Note("d'8")
                Note("g'8")
                Note("a'8")

        ..  container:: example

            Iterates logical voice from second leaf in score:

                >>> container_1 = abjad.Container([
                ...     abjad.Voice("c'8 d'8"),
                ...     abjad.Voice("e'8 f'8"),
                ...     ])
                >>> container_1.is_simultaneous = True
                >>> container_1[0].name = 'Voice 1'
                >>> abjad.override(container_1[0]).stem.direction = abjad.Down
                >>> container_1[1].name = 'Voice 2'
                >>> container_2 = abjad.Container([
                ...     abjad.Voice("g'8 a'8"),
                ...     abjad.Voice("b'8 c''8"),
                ...     ])
                >>> container_2.is_simultaneous = True
                >>> container_2[0].name = 'Voice 1'
                >>> abjad.override(container_2[0]).stem.direction = abjad.Down
                >>> container_2[1].name = 'Voice 2'
                >>> staff = abjad.Staff([container_1, container_2])
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        <<
                            \context Voice = "Voice 1"
                            \with
                            {
                                \override Stem.direction = #down
                            }
                            {
                                c'8
                                d'8
                            }
                            \context Voice = "Voice 2"
                            {
                                e'8
                                f'8
                            }
                        >>
                        <<
                            \context Voice = "Voice 1"
                            \with
                            {
                                \override Stem.direction = #down
                            }
                            {
                                g'8
                                a'8
                            }
                            \context Voice = "Voice 2"
                            {
                                b'8
                                c''8
                            }
                        >>
                    }

            ..  container:: example

                >>> leaf = leaves[1]
                >>> agent = abjad.iterate(leaf)
                >>> for note in agent._logical_voice(
                ...     prototype=abjad.Note,
                ...     ):
                ...     note
                ...
                Note("d'8")
                Note("g'8")
                Note("a'8")

        ..  container:: example

            Iterates all components in logical voice:

            ..  container:: example

                >>> container_1 = abjad.Container([
                ...     abjad.Voice("c'8 d'8"),
                ...     abjad.Voice("e'8 f'8"),
                ...     ])
                >>> container_1.is_simultaneous = True
                >>> container_1[0].name = 'Voice 1'
                >>> abjad.override(container_1[0]).stem.direction = abjad.Down
                >>> container_1[1].name = 'Voice 2'
                >>> container_2 = abjad.Container([
                ...     abjad.Voice("g'8 a'8"),
                ...     abjad.Voice("b'8 c''8"),
                ...     ])
                >>> container_2.is_simultaneous = True
                >>> container_2[0].name = 'Voice 1'
                >>> abjad.override(container_2[0]).stem.direction = abjad.Down
                >>> container_2[1].name = 'Voice 2'
                >>> staff = abjad.Staff([container_1, container_2])
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        <<
                            \context Voice = "Voice 1"
                            \with
                            {
                                \override Stem.direction = #down
                            }
                            {
                                c'8
                                d'8
                            }
                            \context Voice = "Voice 2"
                            {
                                e'8
                                f'8
                            }
                        >>
                        <<
                            \context Voice = "Voice 1"
                            \with
                            {
                                \override Stem.direction = #down
                            }
                            {
                                g'8
                                a'8
                            }
                            \context Voice = "Voice 2"
                            {
                                b'8
                                c''8
                            }
                        >>
                    }

            ..  container:: example

                >>> leaf = leaves[0]
                >>> for component in abjad.iterate(leaf)._logical_voice():
                ...     component
                ...
                Note("c'8")
                Voice("c'8 d'8", name='Voice 1')
                Note("d'8")
                Voice("g'8 a'8", name='Voice 1')
                Note("g'8")
                Note("a'8")

        ..  container:: example

            Iterates all components in logical voice in reverse:

            ..  container:: example

                >>> container_1 = abjad.Container([
                ...     abjad.Voice("c'8 d'8"),
                ...     abjad.Voice("e'8 f'8"),
                ...     ])
                >>> container_1.is_simultaneous = True
                >>> container_1[0].name = 'Voice 1'
                >>> abjad.override(container_1[0]).stem.direction = abjad.Down
                >>> container_1[1].name = 'Voice 2'
                >>> container_2 = abjad.Container([
                ...     abjad.Voice("g'8 a'8"),
                ...     abjad.Voice("b'8 c''8"),
                ...     ])
                >>> container_2.is_simultaneous = True
                >>> container_2[0].name = 'Voice 1'
                >>> abjad.override(container_2[0]).stem.direction = abjad.Down
                >>> container_2[1].name = 'Voice 2'
                >>> staff = abjad.Staff([container_1, container_2])
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        <<
                            \context Voice = "Voice 1"
                            \with
                            {
                                \override Stem.direction = #down
                            }
                            {
                                c'8
                                d'8
                            }
                            \context Voice = "Voice 2"
                            {
                                e'8
                                f'8
                            }
                        >>
                        <<
                            \context Voice = "Voice 1"
                            \with
                            {
                                \override Stem.direction = #down
                            }
                            {
                                g'8
                                a'8
                            }
                            \context Voice = "Voice 2"
                            {
                                b'8
                                c''8
                            }
                        >>
                    }

            ..  container:: example

                >>> leaf = leaves[-1]
                >>> for note in abjad.iterate(leaf)._logical_voice(
                ...     prototype=abjad.Note,
                ...     reverse=True,
                ...     ):
                ...     note
                ...
                Note("c''8")
                Note("b'8")
                Note("f'8")
                Note("e'8")

                >>> leaf = leaves[-1]
                >>> for component in abjad.iterate(leaf)._logical_voice(
                ...     reverse=True,
                ...     ):
                ...     component
                ...
                Note("c''8")
                Voice("b'8 c''8", name='Voice 2')
                Note("b'8")
                Voice("e'8 f'8", name='Voice 2')
                Note("f'8")
                Note("e'8")

        Returns generator.
        """
        import abjad
        prototype = prototype or abjad.Component
        parentage = abjad.inspect(self.client).parentage()
        logical_voice = parentage.logical_voice
        if reverse:
            direction = enums.Right
        else:
            direction = enums.Left
        for component in abjad.iterate(self.client)._depth_first(
            capped=False,
            direction=direction,
            ):
            if not isinstance(component, prototype):
                continue
            parentage = abjad.inspect(component).parentage()
            if parentage.logical_voice == logical_voice:
                yield component

    ### PUBLIC PROPERTIES ###

    @property
    def client(self):
        """
        Gets client.

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.iterate(staff[:2]).client
            Selection([Note("c'4"), Note("d'4")])

        Returns component or selection.
        """
        return self._client

    ### PUBLIC METHODS ###

    def components(self, prototype=None, grace_notes=None, reverse=False):
        r"""
        Iterates components.

        ..  container:: example

            Iterates notes:

            ..  container:: example

                >>> staff = abjad.Staff()
                >>> staff.append(abjad.Measure((2, 8), "c'8 d'8"))
                >>> staff.append(abjad.Measure((2, 8), "e'8 f'8"))
                >>> staff.append(abjad.Measure((2, 8), "g'8 a'8"))
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        {   % measure
                            \time 2/8
                            c'8
                            d'8
                        }   % measure
                        {   % measure
                            e'8
                            f'8
                        }   % measure
                        {   % measure
                            g'8
                            a'8
                        }   % measure
                    }

            ..  container:: example

                >>> agent = abjad.iterate(staff)
                >>> for note in agent.components(prototype=abjad.Note):
                ...     note
                ...
                Note("c'8")
                Note("d'8")
                Note("e'8")
                Note("f'8")
                Note("g'8")
                Note("a'8")

        ..  container:: example

            Iterates grace notes and nongrace notes:

            ..  container:: example

                >>> voice = abjad.Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> container = abjad.GraceContainer("cf''16 bf'16")
                >>> abjad.attach(container, voice[1])
                >>> container = abjad.AfterGraceContainer("af'16 gf'16")
                >>> abjad.attach(container, voice[1])
                >>> abjad.show(voice) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(voice)
                    \new Voice
                    {
                        c'8
                        [
                        \grace {
                            cf''16
                            bf'16
                        }
                        \afterGrace
                        d'8
                        {
                            af'16
                            gf'16
                        }
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for component in abjad.iterate(voice).components():
                ...     component
                ...
                Voice("c'8 d'8 e'8 f'8")
                Note("c'8")
                Note("cf''16")
                Note("bf'16")
                Note("d'8")
                Note("af'16")
                Note("gf'16")
                Note("e'8")
                Note("f'8")

        ..  container:: example

            Iterates components in reverse:

            ..  container:: example

                >>> voice = abjad.Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> container = abjad.GraceContainer("cf''16 bf'16")
                >>> abjad.attach(container, voice[1])
                >>> container = abjad.AfterGraceContainer("af'16 gf'16")
                >>> abjad.attach(container, voice[1])
                >>> abjad.show(voice) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(voice)
                    \new Voice
                    {
                        c'8
                        [
                        \grace {
                            cf''16
                            bf'16
                        }
                        \afterGrace
                        d'8
                        {
                            af'16
                            gf'16
                        }
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for component in abjad.iterate(voice).components(
                ...     reverse=True,
                ...     ):
                ...     component
                ...
                Voice("c'8 d'8 e'8 f'8")
                Note("f'8")
                Note("e'8")
                Note("gf'16")
                Note("af'16")
                Note("d'8")
                Note("bf'16")
                Note("cf''16")
                Note("c'8")

        Returns generator.
        """
        import abjad
        argument = self.client
        prototype = prototype or abjad.Component
        grace_container, after_grace_container = None, None
        if grace_notes is not False and isinstance(argument, abjad.Leaf):
            inspection = abjad.inspect(argument)
            grace_container = inspection.grace_container()
            after_grace_container = inspection.after_grace_container()
        if not reverse:
            if grace_notes is not False and grace_container:
                for component in grace_container:
                    for component_ in abjad.iterate(component).components(
                        prototype,
                        grace_notes=grace_notes,
                        reverse=reverse,
                        ):
                        yield component_
            if isinstance(argument, prototype):
                if (grace_notes is None or
                    (grace_notes is True and
                    abjad.inspect(argument).is_grace_note()) or
                    (grace_notes is False and
                    not abjad.inspect(argument).is_grace_note())):
                    yield argument
            if grace_notes is not False and after_grace_container:
                for component in after_grace_container:
                    for component_ in abjad.iterate(component).components(
                        prototype,
                        grace_notes=grace_notes,
                        reverse=reverse,
                        ):
                        yield component_
            if isinstance(argument, collections.Iterable):
                for item in argument:
                    for component in abjad.iterate(item).components(
                        prototype,
                        grace_notes=grace_notes,
                        reverse=reverse,
                        ):
                        yield component
        else:
            if grace_notes is not False and after_grace_container:
                for component in reversed(after_grace_container):
                    for component_ in abjad.iterate(component).components(
                        prototype,
                        grace_notes=grace_notes,
                        reverse=reverse,
                        ):
                        yield component_
            if isinstance(argument, prototype):
                if (grace_notes is None or
                    (grace_notes is True and
                    abjad.inspect(argument).is_grace_note()) or
                    (grace_notes is False and
                    not abjad.inspect(argument).is_grace_note())):
                    yield argument
            if grace_notes is not False and grace_container:
                for component in reversed(grace_container):
                    for component_ in abjad.iterate(component).components(
                        prototype,
                        grace_notes=grace_notes,
                        reverse=reverse,
                        ):
                        yield component_
            if isinstance(argument, collections.Iterable):
                for item in reversed(argument):
                    for component in abjad.iterate(item).components(
                        prototype,
                        grace_notes=grace_notes,
                        reverse=reverse,
                        ):
                        yield component

    def leaf_pairs(self):
        r"""
        Iterates leaf pairs.

        ..  container:: example

            Iterates leaf pairs:

            ..  container:: example

                >>> score = abjad.Score()
                >>> score.append(abjad.Staff("c'8 d'8 e'8 f'8 g'4"))
                >>> score.append(abjad.Staff("c4 a,4 g,4"))
                >>> abjad.attach(abjad.Clef('bass'), score[1][0])
                >>> abjad.show(score) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(score)
                    \new Score
                    <<
                        \new Staff
                        {
                            c'8
                            d'8
                            e'8
                            f'8
                            g'4
                        }
                        \new Staff
                        {
                            \clef "bass"
                            c4
                            a,4
                            g,4
                        }
                    >>

            ..  container:: example

                >>> for leaf_pair in abjad.iterate(score).leaf_pairs():
                ...     leaf_pair
                ...
                Selection([Note("c'8"), Note('c4')])
                Selection([Note("c'8"), Note("d'8")])
                Selection([Note('c4'), Note("d'8")])
                Selection([Note("d'8"), Note("e'8")])
                Selection([Note("d'8"), Note('a,4')])
                Selection([Note('c4'), Note("e'8")])
                Selection([Note('c4'), Note('a,4')])
                Selection([Note("e'8"), Note('a,4')])
                Selection([Note("e'8"), Note("f'8")])
                Selection([Note('a,4'), Note("f'8")])
                Selection([Note("f'8"), Note("g'4")])
                Selection([Note("f'8"), Note('g,4')])
                Selection([Note('a,4'), Note("g'4")])
                Selection([Note('a,4'), Note('g,4')])
                Selection([Note("g'4"), Note('g,4')])

        Iterates leaf pairs left-to-right and top-to-bottom.

        Returns generator.
        """
        import abjad
        vertical_moments = self.vertical_moments()
        for moment_1, moment_2 in abjad.sequence(vertical_moments).nwise():
            enumerator = abjad.Enumerator(moment_1.start_leaves)
            for pair in enumerator.yield_pairs():
                yield abjad.select(pair)
            sequences = [moment_1.leaves, moment_2.start_leaves]
            enumerator = abjad.Enumerator(sequences)
            for pair in enumerator.yield_outer_product():
                yield abjad.select(pair)
        else:
            enumerator = abjad.Enumerator(moment_2.start_leaves)
            for pair in enumerator.yield_pairs():
                yield abjad.select(pair)

    def leaves(
        self,
        prototype=None,
        grace_notes=None,
        pitched=None,
        reverse=False,
        ):
        r"""
        Iterates leaves.

        ..  container:: example

            Iterates leaves:

            ..  container:: example

                >>> staff = abjad.Staff()
                >>> staff.append(abjad.Measure((2, 8), "<c' bf'>8 <g' a'>8"))
                >>> staff.append(abjad.Measure((2, 8), "af'8 r8"))
                >>> staff.append(abjad.Measure((2, 8), "r8 gf'8"))
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        {   % measure
                            \time 2/8
                            <c' bf'>8
                            <g' a'>8
                        }   % measure
                        {   % measure
                            af'8
                            r8
                        }   % measure
                        {   % measure
                            r8
                            gf'8
                        }   % measure
                    }

            ..  container:: example

                >>> for leaf in abjad.iterate(staff).leaves():
                ...     leaf
                ...
                Chord("<c' bf'>8")
                Chord("<g' a'>8")
                Note("af'8")
                Rest('r8')
                Rest('r8')
                Note("gf'8")

        ..  container:: example

            Iterates grace notes and nongrace notes:

            ..  container:: example

                >>> voice = abjad.Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> container = abjad.GraceContainer("cf''16 bf'16")
                >>> abjad.attach(container, voice[1])
                >>> container = abjad.AfterGraceContainer("af'16 gf'16")
                >>> abjad.attach(container, voice[1])
                >>> abjad.show(voice) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(voice)
                    \new Voice
                    {
                        c'8
                        [
                        \grace {
                            cf''16
                            bf'16
                        }
                        \afterGrace
                        d'8
                        {
                            af'16
                            gf'16
                        }
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for leaf in abjad.iterate(voice).leaves():
                ...     leaf
                ...
                Note("c'8")
                Note("cf''16")
                Note("bf'16")
                Note("d'8")
                Note("af'16")
                Note("gf'16")
                Note("e'8")
                Note("f'8")

        ..  container:: example

            Iterates grace notes only:

            ..  container:: example

                >>> voice = abjad.Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> container = abjad.GraceContainer("cf''16 bf'16")
                >>> abjad.attach(container, voice[1])
                >>> container = abjad.AfterGraceContainer("af'16 gf'16")
                >>> abjad.attach(container, voice[1])
                >>> abjad.show(voice) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(voice)
                    \new Voice
                    {
                        c'8
                        [
                        \grace {
                            cf''16
                            bf'16
                        }
                        \afterGrace
                        d'8
                        {
                            af'16
                            gf'16
                        }
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for leaf in abjad.iterate(voice).leaves(grace_notes=True):
                ...     leaf
                ...
                Note("cf''16")
                Note("bf'16")
                Note("af'16")
                Note("gf'16")

        ..  container:: example

            Iterates nongrace notes only:

            ..  container:: example

                >>> voice = abjad.Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> container = abjad.GraceContainer("cf''16 bf'16")
                >>> abjad.attach(container, voice[1])
                >>> container = abjad.AfterGraceContainer("af'16 gf'16")
                >>> abjad.attach(container, voice[1])
                >>> abjad.show(voice) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(voice)
                    \new Voice
                    {
                        c'8
                        [
                        \grace {
                            cf''16
                            bf'16
                        }
                        \afterGrace
                        d'8
                        {
                            af'16
                            gf'16
                        }
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for leaf in abjad.iterate(voice).leaves(grace_notes=False):
                ...     leaf
                ...
                Note("c'8")
                Note("d'8")
                Note("e'8")
                Note("f'8")

        ..  container:: example

            Iterates pitched leaves only:

            ..  container:: example

                >>> staff = abjad.Staff()
                >>> staff.append(abjad.Measure((2, 8), "<c' bf'>8 <g' a'>8"))
                >>> staff.append(abjad.Measure((2, 8), "af'8 r8"))
                >>> staff.append(abjad.Measure((2, 8), "r8 gf'8"))
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        {   % measure
                            \time 2/8
                            <c' bf'>8
                            <g' a'>8
                        }   % measure
                        {   % measure
                            af'8
                            r8
                        }   % measure
                        {   % measure
                            r8
                            gf'8
                        }   % measure
                    }

            ..  container:: example

                >>> for leaf in abjad.iterate(staff).leaves(pitched=True):
                ...     leaf
                ...
                Chord("<c' bf'>8")
                Chord("<g' a'>8")
                Note("af'8")
                Note("gf'8")

        ..  container:: example

            Iterates nonpitched leaves only:

            ..  container:: example

                >>> staff = abjad.Staff()
                >>> staff.append(abjad.Measure((2, 8), "<c' bf'>8 <g' a'>8"))
                >>> staff.append(abjad.Measure((2, 8), "af'8 r8"))
                >>> staff.append(abjad.Measure((2, 8), "r8 gf'8"))
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        {   % measure
                            \time 2/8
                            <c' bf'>8
                            <g' a'>8
                        }   % measure
                        {   % measure
                            af'8
                            r8
                        }   % measure
                        {   % measure
                            r8
                            gf'8
                        }   % measure
                    }

            ..  container:: example

                >>> for leaf in abjad.iterate(staff).leaves(pitched=False):
                ...     leaf
                ...
                Rest('r8')
                Rest('r8')

        ..  container:: example

            Iterates leaves in reverse:

            ..  container:: example

                >>> staff = abjad.Staff()
                >>> staff.append(abjad.Measure((2, 8), "<c' bf'>8 <g' a'>8"))
                >>> staff.append(abjad.Measure((2, 8), "af'8 r8"))
                >>> staff.append(abjad.Measure((2, 8), "r8 gf'8"))
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        {   % measure
                            \time 2/8
                            <c' bf'>8
                            <g' a'>8
                        }   % measure
                        {   % measure
                            af'8
                            r8
                        }   % measure
                        {   % measure
                            r8
                            gf'8
                        }   % measure
                    }

            ..  container:: example

                >>> for leaf in abjad.iterate(staff).leaves(reverse=True):
                ...     leaf
                ...
                Note("gf'8")
                Rest('r8')
                Rest('r8')
                Note("af'8")
                Chord("<g' a'>8")
                Chord("<c' bf'>8")

        Returns generator.
        """
        import abjad
        prototype = prototype or abjad.Leaf
        if pitched is True:
            prototype = (abjad.Chord, abjad.Note)
        elif pitched is False:
            prototype = (abjad.MultimeasureRest, abjad.Rest, abjad.Skip)
        return self.components(
            prototype=prototype,
            grace_notes=grace_notes,
            reverse=reverse,
            )

    def logical_ties(
        self,
        grace_notes=None,
        nontrivial=None,
        pitched=None,
        reverse=False,
        ):
        r"""
        Iterates logical ties.

        ..  container:: example

            Iterates logical ties:

            ..  container:: example

                >>> string = r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16"
                >>> staff = abjad.Staff(string)
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        c'4
                        ~
                        \times 2/3 {
                            c'16
                            d'8
                        }
                        e'8
                        f'4
                        ~
                        f'16
                    }

            ..  container:: example

                >>> for logical_tie in abjad.iterate(staff).logical_ties():
                ...     logical_tie
                ...
                LogicalTie([Note("c'4"), Note("c'16")])
                LogicalTie([Note("d'8")])
                LogicalTie([Note("e'8")])
                LogicalTie([Note("f'4"), Note("f'16")])

        ..  container:: example

            Iterates pitched logical ties:

            ..  container:: example

                >>> string = r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16"
                >>> staff = abjad.Staff(string)
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        c'4
                        ~
                        \times 2/3 {
                            c'16
                            d'8
                        }
                        e'8
                        f'4
                        ~
                        f'16
                    }

            ..  container:: example

                >>> for logical_tie in abjad.iterate(staff).logical_ties(
                ...     pitched=True,
                ...     ):
                ...     logical_tie
                ...
                LogicalTie([Note("c'4"), Note("c'16")])
                LogicalTie([Note("d'8")])
                LogicalTie([Note("e'8")])
                LogicalTie([Note("f'4"), Note("f'16")])

        ..  container:: example

            Iterates nontrivial logical ties:

            ..  container:: example

                >>> string = r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16"
                >>> staff = abjad.Staff(string)
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        c'4
                        ~
                        \times 2/3 {
                            c'16
                            d'8
                        }
                        e'8
                        f'4
                        ~
                        f'16
                    }

            ..  container:: example

                >>> for logical_tie in abjad.iterate(staff).logical_ties(
                ...     nontrivial=True,
                ...     ):
                ...     logical_tie
                ...
                LogicalTie([Note("c'4"), Note("c'16")])
                LogicalTie([Note("f'4"), Note("f'16")])

        ..  container:: example

            Iterates trivial logical ties:

            ..  container:: example

                >>> string = r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16"
                >>> staff = abjad.Staff(string)
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        c'4
                        ~
                        \times 2/3 {
                            c'16
                            d'8
                        }
                        e'8
                        f'4
                        ~
                        f'16
                    }

            ..  container:: example

                >>> for logical_tie in abjad.iterate(staff).logical_ties(
                ...     nontrivial=False,
                ...     ):
                ...     logical_tie
                ...
                LogicalTie([Note("d'8")])
                LogicalTie([Note("e'8")])

        ..  container:: example

            Iterates logical ties together with grace notes and after grace
            notes:

            ..  container:: example

                >>> voice = abjad.Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> container = abjad.GraceContainer("cf''16 bf'16")
                >>> abjad.attach(container, voice[1])
                >>> container = abjad.AfterGraceContainer("af'16 gf'16")
                >>> abjad.attach(container, voice[1])
                >>> abjad.show(voice) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(voice)
                    \new Voice
                    {
                        c'8
                        [
                        \grace {
                            cf''16
                            bf'16
                        }
                        \afterGrace
                        d'8
                        {
                            af'16
                            gf'16
                        }
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for item in abjad.iterate(voice).logical_ties():
                ...     item
                ...
                LogicalTie([Note("c'8")])
                LogicalTie([Note("cf''16")])
                LogicalTie([Note("bf'16")])
                LogicalTie([Note("d'8")])
                LogicalTie([Note("af'16")])
                LogicalTie([Note("gf'16")])
                LogicalTie([Note("e'8")])
                LogicalTie([Note("f'8")])

        ..  container:: example

            Iterates logical ties in reverse:

            ..  container:: example

                >>> string = r"c'4 ~ \times 2/3 { c'16 d'8 } e'8 f'4 ~ f'16"
                >>> staff = abjad.Staff(string)
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        c'4
                        ~
                        \times 2/3 {
                            c'16
                            d'8
                        }
                        e'8
                        f'4
                        ~
                        f'16
                    }

            ..  container:: example

                >>> for logical_tie in abjad.iterate(staff).logical_ties(
                ...     reverse=True,
                ...     ):
                ...     logical_tie
                ...
                LogicalTie([Note("f'4"), Note("f'16")])
                LogicalTie([Note("e'8")])
                LogicalTie([Note("d'8")])
                LogicalTie([Note("c'4"), Note("c'16")])

        ..  container:: example

            REGRESSION: yields logical tie even when leaves are missing in
            input:

            ..  container:: example

                >>> voice = abjad.Voice("c'8 [ ~ c' ~ c' d' ]")
                >>> abjad.show(voice) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(voice)
                    \new Voice
                    {
                        c'8
                        ~
                        [
                        c'8
                        ~
                        c'8
                        d'8
                        ]
                    }

            ..  container:: example

                >>> selection = voice[:2]
                >>> for logical_tie in abjad.iterate(selection).logical_ties():
                ...     logical_tie
                ...
                LogicalTie([Note("c'8"), Note("c'8"), Note("c'8")])

        Returns generator.
        """
        import abjad
        yielded_logical_ties = set()
        for leaf in self.leaves(
            pitched=pitched,
            grace_notes=grace_notes,
            reverse=reverse,
            ):
            logical_tie = abjad.inspect(leaf).logical_tie()
            if leaf is not logical_tie.head:
                continue
            if (nontrivial is None or
                (nontrivial is True and not logical_tie.is_trivial) or
                (nontrivial is False and logical_tie.is_trivial)):
                if logical_tie not in yielded_logical_ties:
                    yielded_logical_ties.add(logical_tie)
                    yield logical_tie

    def out_of_range(self):
        r"""
        Iterates out-of-range notes and chords.

        ..  container:: example

            >>> staff = abjad.Staff("c'8 r8 <d fs>8 r8")
            >>> violin = abjad.Violin()
            >>> abjad.attach(violin, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'8
                    r8
                    <d fs>8
                    r8
                }

            >>> for leaf in abjad.iterate(staff).out_of_range():
            ...     leaf
            ...
            Chord('<d fs>8')

        Returns generator.
        """
        import abjad
        for leaf in abjad.iterate(self.client).leaves(pitched=True):
            instrument = abjad.inspect(leaf).effective(abjad.Instrument)
            if instrument is None:
                message = 'no instrument found.'
                raise ValueError(message)
            if leaf not in instrument.pitch_range:
                yield leaf

    def pitch_pairs(self):
        r"""
        Iterates pitch pairs.

        ..  container:: example

            Iterates note pitch pairs:

            ..  container:: example

                >>> score = abjad.Score()
                >>> score.append(abjad.Staff("c'8 d' e' f' g'4"))
                >>> score.append(abjad.Staff("c4 a, g,"))
                >>> abjad.attach(abjad.Clef('bass'), score[1][0])
                >>> abjad.show(score) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(score)
                    \new Score
                    <<
                        \new Staff
                        {
                            c'8
                            d'8
                            e'8
                            f'8
                            g'4
                        }
                        \new Staff
                        {
                            \clef "bass"
                            c4
                            a,4
                            g,4
                        }
                    >>

            ..  container:: example

                >>> for pair in abjad.iterate(score).pitch_pairs():
                ...     pair
                ...
                PitchSegment("c' c")
                PitchSegment("c' d'")
                PitchSegment("c d'")
                PitchSegment("d' e'")
                PitchSegment("d' a,")
                PitchSegment("c e'")
                PitchSegment("c a,")
                PitchSegment("e' a,")
                PitchSegment("e' f'")
                PitchSegment("a, f'")
                PitchSegment("f' g'")
                PitchSegment("f' g,")
                PitchSegment("a, g'")
                PitchSegment("a, g,")
                PitchSegment("g' g,")

        ..  container:: example

            Iterates chord pitch pairs:

            ..  container:: example

                >>> staff = abjad.Staff("<c' d' e'>4 <f'' g''>4")

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        <c' d' e'>4
                        <f'' g''>4
                    }

            ..  container:: example

                >>> for pair in abjad.iterate(staff).pitch_pairs():
                ...     pair
                ...
                PitchSegment("c' d'")
                PitchSegment("c' e'")
                PitchSegment("d' e'")
                PitchSegment("c' f''")
                PitchSegment("c' g''")
                PitchSegment("d' f''")
                PitchSegment("d' g''")
                PitchSegment("e' f''")
                PitchSegment("e' g''")
                PitchSegment("f'' g''")

        Returns generator.
        """
        import abjad
        for leaf_pair in self.leaf_pairs():
            pitches = sorted(abjad.iterate(leaf_pair[0]).pitches())
            enumerator = abjad.Enumerator(pitches)
            for pair in enumerator.yield_pairs():
                yield abjad.PitchSegment(pair)
            if isinstance(leaf_pair, set):
                pitches = sorted(abjad.iterate(leaf_pair).pitches())
                enumerator = abjad.Enumerator(pitches)
                for pair in enumerator.yield_pairs():
                    yield abjad.PitchSegment(pair)
            else:
                pitches_1 = sorted(abjad.iterate(leaf_pair[0]).pitches())
                pitches_2 = sorted(abjad.iterate(leaf_pair[1]).pitches())
                sequences = [pitches_1, pitches_2]
                enumerator = abjad.Enumerator(sequences)
                for pair in enumerator.yield_outer_product():
                    yield abjad.PitchSegment(pair)
            pitches = sorted(abjad.iterate(leaf_pair[1]).pitches())
            enumerator = abjad.Enumerator(pitches)
            for pair in enumerator.yield_pairs():
                yield abjad.PitchSegment(pair)

    def pitches(self):
        r"""
        Iterates pitches.

        ..  container:: example

            Iterates pitches in container:

            ..  container:: example

                >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
                >>> beam = abjad.Beam()
                >>> abjad.attach(beam, staff[:])
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        c'8
                        [
                        d'8
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for pitch in abjad.iterate(staff).pitches():
                ...     pitch
                ...
                NamedPitch("c'")
                NamedPitch("d'")
                NamedPitch("e'")
                NamedPitch("f'")

        ..  container:: example

            Iterates pitches in spanner:

            ..  container:: example

                >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
                >>> beam = abjad.Beam()
                >>> abjad.attach(beam, staff[:])
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        c'8
                        [
                        d'8
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for pitch in abjad.iterate(beam).pitches():
                ...     pitch
                ...
                NamedPitch("c'")
                NamedPitch("d'")
                NamedPitch("e'")
                NamedPitch("f'")

        ..  container:: example

            Iterates pitches in pitch set:

            ..  container:: example

                >>> pitch_set = abjad.PitchSet([0, 2, 4, 5])

            ..  container:: example

                >>> for pitch in abjad.iterate(pitch_set).pitches():
                ...     pitch
                ...
                NumberedPitch(0)
                NumberedPitch(2)
                NumberedPitch(4)
                NumberedPitch(5)

        ..  container:: example

            Iterates different types of object in tuple:

            ..  container:: example

                >>> pitches = (
                ...     abjad.NamedPitch("c'"),
                ...     abjad.Note("d'4"),
                ...     abjad.Chord("<e' fs' g>4"),
                ...     )

            ..  container:: example

                >>> for pitch in abjad.iterate(pitches).pitches():
                ...     pitch
                ...
                NamedPitch("c'")
                NamedPitch("d'")
                NamedPitch('g')
                NamedPitch("e'")
                NamedPitch("fs'")

        Returns generator.
        """
        import abjad
        if isinstance(self.client, abjad.Pitch):
            pitch = abjad.NamedPitch(self.client)
            yield pitch
        result = []
        try:
            result.extend(self.client.pitches)
        except AttributeError:
            pass
        if isinstance(self.client, abjad.Chord):
            result.extend(self.client.written_pitches)
        elif isinstance(self.client, abjad.Spanner):
            for leaf in self.client.leaves:
                try:
                    result.append(leaf.written_pitch)
                except AttributeError:
                    pass
                try:
                    result.extedn(leaf.written_pitches)
                except AttributeError:
                    pass
        elif isinstance(self.client, abjad.PitchSet):
            result.extend(sorted(list(self.client)))
        elif isinstance(self.client, (list, tuple, set)):
            for item in self.client:
                for pitch_ in abjad.iterate(item).pitches():
                    result.append(pitch_)
        else:
            for leaf in abjad.iterate(self.client).leaves():
                try:
                    result.append(leaf.written_pitch)
                except AttributeError:
                    pass
                try:
                    result.extedn(leaf.written_pitches)
                except AttributeError:
                    pass
        for pitch in result:
            yield pitch

    def spanners(self, prototype=None, reverse=False):
        r"""
        Iterates spanners.

        ..  container:: example

            Iterates spanners:

            ..  container:: example

                >>> staff = abjad.Staff("c'8 d'8 e'8 f'8 g'8 a'8 f'8 b'8 c''8")
                >>> abjad.attach(abjad.Slur(), staff[:4])
                >>> abjad.attach(abjad.Slur(), staff[4:])
                >>> abjad.attach(abjad.Beam(), staff[:])
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        c'8
                        [
                        (
                        d'8
                        e'8
                        f'8
                        )
                        g'8
                        (
                        a'8
                        f'8
                        b'8
                        c''8
                        ]
                        )
                    }

            ..  container:: example

                >>> for spanner in abjad.iterate(staff).spanners():
                ...     spanner
                ...
                Beam("c'8, d'8, ... [5] ..., b'8, c''8", durations=(), span_beam_count=1)
                Slur("c'8, d'8, e'8, f'8")
                Slur("g'8, a'8, f'8, b'8, c''8")

        ..  container:: example

            Iterates spanners in reverse:

            ..  container:: example

                >>> staff = abjad.Staff("c'8 d'8 e'8 f'8 g'8 a'8 f'8 b'8 c''8")
                >>> abjad.attach(abjad.Slur(), staff[:4])
                >>> abjad.attach(abjad.Slur(), staff[4:])
                >>> abjad.attach(abjad.Beam(), staff[:])
                >>> abjad.show(staff) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(staff)
                    \new Staff
                    {
                        c'8
                        [
                        (
                        d'8
                        e'8
                        f'8
                        )
                        g'8
                        (
                        a'8
                        f'8
                        b'8
                        c''8
                        ]
                        )
                    }

            ..  container:: example

                >>> for spanner in abjad.iterate(staff).spanners(reverse=True):
                ...     spanner
                ...
                Beam("c'8, d'8, ... [5] ..., b'8, c''8", durations=(), span_beam_count=1)
                Slur("g'8, a'8, f'8, b'8, c''8")
                Slur("c'8, d'8, e'8, f'8")

        Returns generator.
        """
        import abjad
        visited_spanners = set()
        for component in self.components(reverse=reverse):
            spanners = abjad.inspect(component).spanners(
                prototype=prototype,
                )
            spanners = sorted(spanners,
                key=lambda x: (
                    type(x).__name__,
                    abjad.inspect(x).timespan(),
                    ),
                )
            for spanner in spanners:
                if spanner in visited_spanners:
                    continue
                visited_spanners.add(spanner)
                yield spanner

    def timeline(self, prototype=None, reverse=False):
        r"""
        Iterates timeline.

        ..  container:: example

            Timeline-iterates leaves:

            ..  container:: example

                >>> score = abjad.Score()
                >>> score.append(abjad.Staff("c'4 d'4 e'4 f'4"))
                >>> score.append(abjad.Staff("g'8 a'8 b'8 c''8"))
                >>> abjad.show(score) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(score)
                    \new Score
                    <<
                        \new Staff
                        {
                            c'4
                            d'4
                            e'4
                            f'4
                        }
                        \new Staff
                        {
                            g'8
                            a'8
                            b'8
                            c''8
                        }
                    >>

            ..  container:: example

                >>> for leaf in abjad.iterate(score).timeline():
                ...     leaf
                ...
                Note("c'4")
                Note("g'8")
                Note("a'8")
                Note("d'4")
                Note("b'8")
                Note("c''8")
                Note("e'4")
                Note("f'4")

        ..  container:: example

            Timeline-iterates leaves in reverse:

            ..  container:: example

                >>> score = abjad.Score()
                >>> score.append(abjad.Staff("c'4 d'4 e'4 f'4"))
                >>> score.append(abjad.Staff("g'8 a'8 b'8 c''8"))
                >>> abjad.show(score) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(score)
                    \new Score
                    <<
                        \new Staff
                        {
                            c'4
                            d'4
                            e'4
                            f'4
                        }
                        \new Staff
                        {
                            g'8
                            a'8
                            b'8
                            c''8
                        }
                    >>

            ..  container:: example

                >>> for leaf in abjad.iterate(score).timeline(reverse=True):
                ...     leaf
                ...
                Note("f'4")
                Note("e'4")
                Note("d'4")
                Note("c''8")
                Note("b'8")
                Note("c'4")
                Note("a'8")
                Note("g'8")

        ..  container:: example

            Timeline-iterates leaves together grace notes:

            ..  container:: example

                >>> voice = abjad.Voice("c'8 [ d'8 e'8 f'8 ]")
                >>> container = abjad.GraceContainer("cf''16 bf'16")
                >>> abjad.attach(container, voice[1])
                >>> abjad.show(voice) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(voice)
                    \new Voice
                    {
                        c'8
                        [
                        \grace {
                            cf''16
                            bf'16
                        }
                        d'8
                        e'8
                        f'8
                        ]
                    }

            ..  container:: example

                >>> for component in abjad.iterate(voice).timeline():
                ...     component
                ...
                Note("c'8")
                Note("d'8")
                Note("e'8")
                Note("f'8")

                ..  todo:: Incorrect because grace notes are not included.

        Iterates leaves when ``prototype`` is none.
        """
        import abjad
        prototype = prototype or abjad.Leaf
        if isinstance(self.client, abjad.Component):
            components = [self.client]
        else:
            components = list(self.client)
        if not reverse:
            while components:
                current_start_offset = min(
                    abjad.inspect(_).timespan().start_offset
                    for _ in components
                    )
                components.sort(
                    key=lambda x: abjad.inspect(x).parentage(
                        grace_notes=True).score_index,
                    reverse=True,
                    )
                components_to_process = components[:]
                components = []
                while components_to_process:
                    component = components_to_process.pop()
                    start_offset = abjad.inspect(component).timespan().start_offset
                    #print('    COMPONENT:', component)
                    if current_start_offset < start_offset:
                        components.append(component)
                        #print('        TOO EARLY')
                        continue
                    if isinstance(component, prototype):
                        #print('        YIELDING', component)
                        yield component
                    sibling = component._get_sibling(1)
                    if sibling is not None:
                        #print('        SIBLING:', sibling)
                        components.append(sibling)
                    if not isinstance(component, abjad.Container):
                        continue
                    if not len(component):
                        continue
                    if not component.is_simultaneous:
                        components_to_process.append(component[0])
                    else:
                        components_to_process.extend(reversed(component))
        else:
            while components:
                #print('STEP')
                #print()
                current_stop_offset = max(
                    abjad.inspect(_).timespan().stop_offset
                    for _ in components
                    )
                components.sort(
                    key=lambda x: abjad.inspect(x).parentage(
                        grace_notes=True).score_index,
                    reverse=True,
                    )
                components_to_process = components[:]
                components = []
                while components_to_process:
                    component = components_to_process.pop()
                    stop_offset = abjad.inspect(component).timespan().stop_offset
                    #print('\tCOMPONENT:', component)
                    if stop_offset < current_stop_offset:
                        components.insert(0, component)
                        continue
                    if isinstance(component, prototype):
                        yield component
                    sibling = component._get_sibling(-1)
                    if sibling is not None:
                        components.insert(0, sibling)
                    if not isinstance(component, abjad.Container):
                        continue
                    if not len(component):
                        continue
                    if not component.is_simultaneous:
                        components_to_process.append(component[-1])
                    else:
                        components_to_process.extend(reversed(component))

    def vertical_moments(self, reverse=False):
        r'''Iterates vertical moments.

        ..  container:: example

            Iterates vertical moments:

            ..  container:: example

                >>> score = abjad.Score([])
                >>> staff = abjad.Staff(r"\times 4/3 { d''8 c''8 b'8 }")
                >>> score.append(staff)
                >>> staff_group = abjad.StaffGroup([])
                >>> staff_group.lilypond_type = 'PianoStaff'
                >>> staff_group.append(abjad.Staff("a'4 g'4"))
                >>> staff_group.append(abjad.Staff(r"""\clef "bass" f'8 e'8 d'8 c'8"""))
                >>> score.append(staff_group)
                >>> abjad.show(score) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(score)
                    \new Score
                    <<
                        \new Staff
                        {
                            \tweak text #tuplet-number::calc-fraction-text
                            \times 4/3 {
                                d''8
                                c''8
                                b'8
                            }
                        }
                        \new PianoStaff
                        <<
                            \new Staff
                            {
                                a'4
                                g'4
                            }
                            \new Staff
                            {
                                \clef "bass"
                                f'8
                                e'8
                                d'8
                                c'8
                            }
                        >>
                    >>

            ..  container:: example

                >>> for vertical_moment in abjad.iterate(score).vertical_moments():
                ...     vertical_moment.leaves
                ...
                Selection([Note("d''8"), Note("a'4"), Note("f'8")])
                Selection([Note("d''8"), Note("a'4"), Note("e'8")])
                Selection([Note("c''8"), Note("a'4"), Note("e'8")])
                Selection([Note("c''8"), Note("g'4"), Note("d'8")])
                Selection([Note("b'8"), Note("g'4"), Note("d'8")])
                Selection([Note("b'8"), Note("g'4"), Note("c'8")])

                >>> for vertical_moment in abjad.iterate(staff_group).vertical_moments():
                ...     vertical_moment.leaves
                ...
                Selection([Note("a'4"), Note("f'8")])
                Selection([Note("a'4"), Note("e'8")])
                Selection([Note("g'4"), Note("d'8")])
                Selection([Note("g'4"), Note("c'8")])

        ..  container:: example

            Iterates vertical moments in reverse:

            ..  container:: example

                >>> score = abjad.Score([])
                >>> staff = abjad.Staff(r"\times 4/3 { d''8 c''8 b'8 }")
                >>> score.append(staff)
                >>> staff_group = abjad.StaffGroup([])
                >>> staff_group.lilypond_type = 'PianoStaff'
                >>> staff_group.append(abjad.Staff("a'4 g'4"))
                >>> staff_group.append(abjad.Staff(r"""\clef "bass" f'8 e'8 d'8 c'8"""))
                >>> score.append(staff_group)
                >>> abjad.show(score) # doctest: +SKIP

                ..  docs::

                    >>> abjad.f(score)
                    \new Score
                    <<
                        \new Staff
                        {
                            \tweak text #tuplet-number::calc-fraction-text
                            \times 4/3 {
                                d''8
                                c''8
                                b'8
                            }
                        }
                        \new PianoStaff
                        <<
                            \new Staff
                            {
                                a'4
                                g'4
                            }
                            \new Staff
                            {
                                \clef "bass"
                                f'8
                                e'8
                                d'8
                                c'8
                            }
                        >>
                    >>

            ..  container:: example

                >>> agent = abjad.iterate(score)
                >>> for vertical_moment in agent.vertical_moments(
                ...     reverse=True,
                ...     ):
                ...     vertical_moment.leaves
                ...
                Selection([Note("b'8"), Note("g'4"), Note("c'8")])
                Selection([Note("b'8"), Note("g'4"), Note("d'8")])
                Selection([Note("c''8"), Note("g'4"), Note("d'8")])
                Selection([Note("c''8"), Note("a'4"), Note("e'8")])
                Selection([Note("d''8"), Note("a'4"), Note("e'8")])
                Selection([Note("d''8"), Note("a'4"), Note("f'8")])

                >>> agent = abjad.iterate(staff_group)
                >>> for vertical_moment in agent.vertical_moments(
                ...     reverse=True,
                ...     ):
                ...     vertical_moment.leaves
                ...
                Selection([Note("g'4"), Note("c'8")])
                Selection([Note("g'4"), Note("d'8")])
                Selection([Note("a'4"), Note("e'8")])
                Selection([Note("a'4"), Note("f'8")])

        Returns generator.
        '''
        import abjad
        def _buffer_components_starting_with(component, buffer, stop_offsets):
            buffer.append(component)
            stop_offset = abjad.inspect(component).timespan().stop_offset
            stop_offsets.append(stop_offset)
            if isinstance(component, abjad.Container):
                if component.is_simultaneous:
                    for component_ in component:
                        _buffer_components_starting_with(
                            component_,
                            buffer,
                            stop_offsets,
                            )
                elif component:
                    _buffer_components_starting_with(
                        component[0],
                        buffer,
                        stop_offsets,
                        )
        def _iterate_vertical_moments(argument):
            governors = (argument,)
            current_offset, stop_offsets, buffer = abjad.Offset(0), [], []
            _buffer_components_starting_with(argument, buffer, stop_offsets)
            while buffer:
                vertical_moment = abjad.VerticalMoment()
                offset = abjad.Offset(current_offset)
                components = list(buffer)
                components.sort(
                    key=lambda _: abjad.inspect(_).parentage().score_index
                    )
                vertical_moment._offset = offset
                vertical_moment._governors = governors
                vertical_moment._components = components
                yield vertical_moment
                current_offset, stop_offsets = min(stop_offsets), []
                _update_buffer(current_offset, buffer, stop_offsets)
        def _next_in_parent(component):
            assert isinstance(component, abjad.Component), repr(component)
            selection = abjad.select(component)
            result = selection._get_parent_and_start_stop_indices()
            parent, start, stop = result
            assert start == stop
            if parent is None:
                raise StopIteration
            if parent.is_simultaneous:
                raise StopIteration
            try:
                return parent[start + 1]
            except IndexError:
                raise StopIteration
        def _update_buffer(current_offset, buffer, stop_offsets):
            for component in buffer[:]:
                offset = abjad.inspect(component).timespan().stop_offset
                if offset <= current_offset:
                    buffer.remove(component)
                    try:
                        next_component = _next_in_parent(component)
                        _buffer_components_starting_with(
                            next_component,
                            buffer,
                            stop_offsets,
                            )
                    except StopIteration:
                        pass
                else:
                    stop_offsets.append(offset)
        if not reverse:
            for x in _iterate_vertical_moments(self.client):
                yield x
        else:
            moments_in_governor = []
            for component in self.components():
                offset = abjad.inspect(component).timespan().start_offset
                if offset not in moments_in_governor:
                    moments_in_governor.append(offset)
            moments_in_governor.sort()
            for moment_in_governor in reversed(moments_in_governor):
                yield self.client._get_vertical_moment_at(moment_in_governor)



import collections
import importlib
import types
import uqbar.apis


def yield_all_modules(paths=None):
    '''
    Yields all modules encountered in `path`.

    Returns generator.
    '''
    import abjad
    _paths = []
    if not paths:
        _paths = abjad.__path__
    elif isinstance(paths, str):
        module = importlib.import_module(paths)
        _paths.extend(module.__path__)
    elif isinstance(paths, types.ModuleType):
        _paths.extend(paths.__path__)
    elif isinstance(paths, collections.Iterable):
        for path in paths:
            if isinstance(path, types.ModuleType):
                _paths.extend(path.__path__)
            elif isinstance(path, str):
                module = importlib.import_module(path)
                _paths.extend(module.__path__)
            else:
                raise ValueError(module)
    for path in _paths:
        for source_path in uqbar.apis.collect_source_paths([path]):
            package_path = uqbar.apis.source_path_to_package_path(source_path)
            yield importlib.import_module(package_path)



import importlib
import os
import subprocess
import sys
import traceback
from abjad.system.Configuration import Configuration


class AbjadConfiguration(Configuration):
    """
    Abjad configuration.

    ..  container:: example

        >>> abjad_configuration = abjad.AbjadConfiguration()

    ..  container:: example

        Behavior at instantiation:

        * Looks for ``$HOME/.abjad/``.

        * Creates ``$HOME/.abjad/`` if directory does not exist.

        * Looks for ``$HOME/.abjad/abjad.cfg``.

        * Creates ``$HOME/.abjad/abjad.cfg`` if file does not exist.

        * Parses ``$HOME/.abjad/abjad.cfg``.

        * Provides default key-value pairs for pairs which fail validation.

        * Writes configuration changes to disk.

        * Creates Abjad output directory if directory does not exist.

    Supports mutable mapping dictionary interface.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'System configuration'

    __slots__ = (
        '_composer_library_tools',
        )

    _configuration_directory_name = '.abjad'

    _configuration_file_name = 'abjad.cfg'

    # for caching
    _lilypond_version_string = None

    ### INITIALIZER ###

    def __init__(self):
        Configuration.__init__(self)
        if not os.path.exists(self.abjad_output_directory):
            try:
                os.makedirs(self.abjad_output_directory)
            except (IOError, OSError):
                traceback.print_exc()
        self._composer_library_tools = None

    ### PRIVATE METHODS ###

    def _get_initial_comment(self):
        current_time = self._get_current_time()
        return [
            'Abjad configuration file created on {}.'.format(current_time),
            "This file is interpreted by Python's ConfigParser ",
            'and follows ini syntax.',
            ]

    def _get_option_definitions(self):
        options = {
            'abjad_output_directory': {
                'comment': [
                    'Set to the directory where all Abjad-generated files',
                    '(such as PDFs and LilyPond files) should be saved.',
                    'Defaults to $HOME/.abjad/output/'
                    ],
                'default': os.path.join(
                    str(self.configuration_directory),
                    'output',
                    ),
                'validator': str,
                },
            'composer_email': {
                'comment': ['Your email.'],
                'default': 'first.last@domain.com',
                'validator': str,
                },
            'composer_full_name': {
                'comment': ['Your full name.'],
                'default': 'Full Name',
                'validator': str,
                },
            'composer_github_username': {
                'comment': ['Your GitHub username.'],
                'default': 'username',
                'validator': str,
                },
            'composer_last_name': {
                'comment': ['Your last name.'],
                'default': 'Name',
                'validator': str,
                },
            'composer_library': {
                'comment': ['Your library.'],
                'default': 'my_library',
                'validator': str,
                },
            'composer_scores_directory': {
                'comment': ['Your scores directory.'],
                'default': str(self.home_directory / 'scores'),
                'validator': str,
                },
            'composer_uppercase_name': {
                'comment': ['Your full name in uppercase for score covers.'],
                'default': 'FULL NAME',
                'validator': str,
                },
            'composer_website': {
                'comment': ['Your website.'],
                'default': 'www.composername.com',
                'validator': str,
                },
            'lilypond_path': {
                'comment': [
                    'Lilypond executable path. Set to override dynamic lookup.'
                    ],
                'default': 'lilypond',
                'validator': str,
                },
            'midi_player': {
                'comment': [
                    'MIDI player to open MIDI files.',
                    'When unset your OS should know how to open MIDI files.',
                    ],
                'default': None,
                'validator': str,
                },
            'pdf_viewer': {
                'comment': [
                    'PDF viewer to open PDF files.',
                    'When unset your OS should know how to open PDFs.',
                    ],
                'default': None,
                'validator': str,
                },
            'text_editor': {
                'comment': [
                    'Text editor to edit text files.',
                    'When unset your OS should know how to open text files.'
                    ],
                'default': None,
                'validator': str,
                },
            }
        return options

    ### PUBLIC PROPERTIES ###

    @property
    def abjad_directory(self):
        """
        Gets Abjad directory.

        Returns string.
        """
        import abjad
        return abjad.__path__[0]

    @property
    def abjad_output_directory(self):
        """
        Gets Abjad output directory.

        Returns string.
        """
        if 'abjad_output_directory' in self._settings:
            return self._settings['abjad_output_directory']
        return os.path.join(
            self.configuration_directory,
            'output'
            )

    @property
    def abjad_root_directory(self):
        """
        Gets Abjad root directory.

        Returns string.
        """
        relative_path = os.path.join(
            self.abjad_directory,
            '..',
            )
        return os.path.abspath(relative_path)

    @property
    def boilerplate_directory(self):
        """
        Gets Abjad boilerplate directory.

        Return string.
        """
        relative_path = os.path.join(
            self.abjad_directory,
            'boilerplate',
            )
        return os.path.abspath(relative_path)

    @property
    def composer_email(self):
        """
        Gets composer email.

        Returns string.
        """
        return self._settings['composer_email']

    @property
    def composer_full_name(self):
        """
        Gets composer full name.

        Returns string.
        """
        return self._settings['composer_full_name']

    @property
    def composer_github_username(self):
        """
        Gets GitHub username.

        Returns string.
        """
        return self._settings['composer_github_username']

    @property
    def composer_last_name(self):
        """
        Gets composer last name.

        Returns string.
        """
        return self._settings['composer_last_name']

    @property
    def composer_library(self):
        """
        Gets composer library package name.

        Returns string.
        """
        return self._settings['composer_library']

    @property
    def composer_library_tools(self):
        """
        Gets composer library tools directory.

        Returns string.
        """
        if self._composer_library_tools is None:
            name = self.composer_library
            if not name:
                return
            try:
                path = importlib.import_module(name)
            except ImportError:
                path = None
            if not path:
                return
            #path = os.path.join(path.__path__[0], 'tools')
            path = path.__path__[0]
            self._composer_library_tools = path
        return self._composer_library_tools

    @property
    def composer_scores_directory(self):
        """
        Gets composer scores directory.

        Returns string.
        """
        if 'composer_scores_directory' in self._settings:
            return self._settings['composer_scores_directory']
        return os.path.join(self.home_directory, 'scores')

    @property
    def composer_uppercase_name(self):
        """
        Gets composer uppercase name.

        ..  container:: example

            >>> configuration.composer_uppercase_name # doctest: +SKIP
            'TREVOR BAČA'

        Returns string.
        """
        return self._settings['composer_uppercase_name']

    @property
    def composer_website(self):
        """
        Gets composer website.

        ..  container:: example

            >>> configuration.composer_website  # doctest: +SKIP
            'www.trevobaca.com'

        Returns string.
        """
        return self._settings['composer_website']

    @property
    def lilypond_log_file_path(self):
        """
        Gets LilyPond log file path.

        Returns string.
        """
        return os.path.join(self.abjad_output_directory, 'lily.log')

    ### PUBLIC METHODS ###

    @classmethod
    def get_abjad_startup_string(class_):
        """
        Gets Abjad startup string.

        ..  container:: example

            >>> abjad_configuration = abjad.AbjadConfiguration()
            >>> abjad_configuration.get_abjad_startup_string()
            'Abjad 3.0.0rc1 (development)'

        Returns string.
        """
        result = 'Abjad {} ({})'
        result = result.format(
            class_.get_abjad_version_string(),
            'development',
            )
        return result

    @staticmethod
    def get_abjad_version_string():
        """
        Gets Abjad version string.

        ..  container:: example

            >>> abjad_configuration = abjad.AbjadConfiguration()
            >>> abjad_configuration.get_abjad_version_string()
            '3.0.0rc1'

        Returns string.
        """
        import abjad
        return abjad.__version__

    @classmethod
    def get_lilypond_minimum_version_string(class_):
        """
        Gets LilyPond minimum version string.

        ..  container:: example

            >>> abjad_configuration = abjad.AbjadConfiguration()
            >>> abjad_configuration.get_lilypond_minimum_version_string() # doctest: +SKIP
            '2.19.0'

        Returns string.
        """
        version = class_.get_lilypond_version_string()
        parts = version.split('.')[0:2]
        parts.append('0')
        return '.'.join(parts)

    @staticmethod
    def get_lilypond_version_string():
        """
        Gets LilyPond version string.

        ..  container:: example

            >>> abjad_configuration = abjad.AbjadConfiguration()
            >>> abjad_configuration.get_lilypond_version_string() # doctest: +SKIP
            '2.19.82'

        Returns string.
        """
        from abjad import abjad_configuration
        from abjad import system
        if AbjadConfiguration._lilypond_version_string is not None:
            return AbjadConfiguration._lilypond_version_string
        lilypond = abjad_configuration.get('lilypond_path')
        if not lilypond:
            lilypond = system.IOManager.find_executable('lilypond')
            if lilypond:
                lilypond = lilypond[0]
            else:
                lilypond = 'lilypond'
        command = lilypond + ' --version'
        proc = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
        lilypond_version_string = proc.stdout.readline().decode()
        lilypond_version_string = lilypond_version_string.split(' ')[-1]
        lilypond_version_string = lilypond_version_string.strip()
        AbjadConfiguration._lilypond_version_string = lilypond_version_string
        return lilypond_version_string

    @staticmethod
    def get_python_version_string():
        """
        Gets Python version string.

        ..  container:: example

            >>> abjad_configuration = abjad.AbjadConfiguration()
            >>> abjad_configuration.get_python_version_string() # doctest: +SKIP
            '3.6.4'

        Returns string.
        """
        return '.'.join(str(_) for _ in sys.version_info[:3])

    @staticmethod
    def get_tab_width():
        """
        Gets tab width.

        ..  container:: example

            >>> abjad_configuration = abjad.AbjadConfiguration()
            >>> abjad_configuration.get_tab_width()
            4

        Used by code generation functions.

        Returns nonnegative integer.
        """
        return 4

    @staticmethod
    def get_text_editor():
        """
        Gets text editor.

        ..  container:: example

            >>> abjad_configuration = abjad.AbjadConfiguration()
            >>> abjad_configuration.get_text_editor() # doctest: +SKIP
            'vi'

        Returns string.
        """
        from abjad import abjad_configuration
        text_editor = abjad_configuration['text_editor']
        if text_editor is not None:
            return text_editor
        elif os.name == 'posix':
            return 'vi'
        else:
            return 'edit'

    @staticmethod
    def list_package_dependency_versions():
        """
        Lists package dependency versions.

        ..  container:: example

            >>> abjad_configuration = abjad.AbjadConfiguration()
            >>> abjad_configuration.list_package_dependency_versions() # doctest: +SKIP
            {'sphinx': '1.1.2', 'pytest': '2.1.2'}

        Returns dictionary.
        """
        dependencies = {}
        dependencies['configobj'] = None
        try:
            import configobj  # type: ignore
            dependencies['configobj'] = configobj.__version__
        except (AttributeError, ImportError):
            pass
        dependencies['ply'] = None
        try:
            from ply import lex  # type: ignore
            dependencies['ply'] = lex.__version__
        except ImportError:
            pass
        dependencies['pytest'] = None
        try:
            import pytest  # type: ignore
            dependencies['pytest'] = pytest.__version__
        except ImportError:
            pass
        dependencies['sphinx'] = None
        try:
            import sphinx  # type: ignore
            dependencies['sphinx'] = sphinx.__version__
        except ImportError:
            pass
        return dependencies



import typing
from abjad import enums
from abjad.lilypondnames.LilyPondGrobOverride import LilyPondGrobOverride
from abjad.lilypondnames.LilyPondTweakManager import LilyPondTweakManager
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle
from abjad.system.LilyPondFormatManager import LilyPondFormatManager
from abjad.system.Tags import Tags
abjad_tags = Tags()


class DynamicTrend(AbjadValueObject):
    r"""
    Dynamic trend.

    ..  container:: example

        >>> staff = abjad.Staff("c'4 d' e' f'")
        >>> abjad.attach(abjad.Dynamic('p'), staff[0])
        >>> abjad.attach(abjad.DynamicTrend('<'), staff[0])
        >>> abjad.attach(abjad.Dynamic('f'), staff[-1])
        >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                \override DynamicLineSpanner.staff-padding = #4.5
            }
            {
                c'4
                \p
                \<
                d'4
                e'4
                f'4
                \f
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_left_broken',
        '_shape',
        '_tweaks',
        )

    _context = 'Voice'

    _crescendo_start = r'\<'

    _decrescendo_start = r'\>'

    _format_slot = 'after'

    _known_shapes = (
        '<', 'o<', '<|', 'o<|',
        '>', '>o', '|>', '|>o',
        '--',
        )

    _time_orientation: enums.HorizontalAlignment = enums.Right

    ### INITIALIZER ###

    def __init__(
        self,
        shape='<',
        *,
        left_broken: bool = None,
        tweaks: typing.Union[
            typing.List[typing.Tuple], LilyPondTweakManager] = None,
        ) -> None:
        if left_broken is not None:
            left_broken = bool(left_broken)
        self._left_broken = left_broken
        assert shape in self._known_shapes, repr(shape)
        self._shape = shape
        self._tweaks = None
        LilyPondTweakManager.set_tweaks(self, tweaks)

    ### PRIVATE METHODS ###

    @staticmethod
    def _constante_hairpin():
        return LilyPondGrobOverride(
            grob_name='Hairpin',
            once=True,
            property_path='stencil',
            value='#constante-hairpin',
            )

    @staticmethod
    def _circled_tip():
        return LilyPondGrobOverride(
            grob_name='Hairpin',
            once=True,
            property_path='circled-tip',
            value=True,
            )

    @staticmethod
    def _flared_hairpin():
        return LilyPondGrobOverride(
            grob_name='Hairpin',
            once=True,
            property_path='stencil',
            value='#abjad-flared-hairpin',
            )

    def _get_lilypond_format(self):
        strings = []
        if '--' in self.shape:
            override = self._constante_hairpin()
            string = override.tweak_string()
            strings.append(string)
        if 'o' in self.shape:
            override = self._circled_tip()
            string = override.tweak_string()
            strings.append(string)
        if '|' in self.shape:
            override = self._flared_hairpin()
            string = override.tweak_string()
            strings.append(string)
        if '<' in self.shape or '--' in self.shape:
            strings.append(self._crescendo_start)
        elif '>' in self.shape:
            strings.append(self._decrescendo_start)
        else:
            raise ValueError(self.shape)
        if self.left_broken is True:
            strings = self._tag_hide(strings)
        return strings

    def _get_lilypond_format_bundle(self, component=None):
        """
        Dynamic trend contributes formatting to the 'spanners' slot
        (rather than the 'commands' slot). The reason for this is that
        the LilyPond \startTrillSpan [pitch] command must appear after
        \< and \> but before \set and other commmands.
        """
        bundle = LilyPondFormatBundle()
        if self.tweaks:
            tweaks = self.tweaks._list_format_contributions()
            bundle.after.spanners.extend(tweaks)
        strings = self._get_lilypond_format()
        bundle.after.spanners.extend(strings)
        return bundle

    @staticmethod
    def _tag_hide(strings):
        import abjad
        abjad_tags = abjad.Tags()
        return LilyPondFormatManager.tag(
            strings,
            deactivate=False,
            tag=abjad_tags.HIDE_TO_JOIN_BROKEN_SPANNERS,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def context(self) -> str:
        r"""
        Returns (historically conventional) context ``'Voice'``.

        ..  container:: example

            >>> abjad.DynamicTrend('<').context
            'Voice'

        ..  container:: example

            >>> voice = abjad.Voice("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('p'), voice[0])
            >>> abjad.attach(abjad.DynamicTrend('<'), voice[0])
            >>> abjad.attach(abjad.Dynamic('f'), voice[-1])
            >>> abjad.show(voice) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(voice)
                \new Voice
                {
                    c'4
                    \p
                    \<
                    d'4
                    e'4
                    f'4
                    \f
                }


            >>> for leaf in voice:
            ...     print(leaf, abjad.inspect(leaf).effective(abjad.DynamicTrend))
            c'4 DynamicTrend(shape='<')
            d'4 DynamicTrend(shape='<')
            e'4 DynamicTrend(shape='<')
            f'4 DynamicTrend(shape='<')

        Class constant.

        Override with ``abjad.attach(..., context='...')``.
        """
        return self._context

    @property
    def known_shapes(self) -> typing.Tuple[str, ...]:
        r"""

        Gets known shapes.

        ..  container:: example

            >>> for shape in abjad.DynamicTrend().known_shapes:
            ...     shape
            '<'
            'o<'
            '<|'
            'o<|'
            '>'
            '>o'
            '|>'
            '|>o'
            '--'

        """
        return self._known_shapes

    @property
    def left_broken(self) -> typing.Optional[bool]:
        r"""
        Is true when dynamic trend formats with left broken tag.

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> trend = abjad.DynamicTrend('<', left_broken=True)
            >>> stop = abjad.Dynamic('f')
            >>> abjad.attach(trend, staff[0])
            >>> abjad.attach(stop, staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            >>> abjad.f(staff)
            \new Staff
            \with
            {
                \override DynamicLineSpanner.staff-padding = #4.5
            }
            {
                c'4
                \< %! HIDE_TO_JOIN_BROKEN_SPANNERS
                d'4
                e'4
                f'4
                \f
            }

        """
        return self._left_broken

    @property
    def shape(self) -> str:
        r"""
        Gets shape.

        ..  container:: example

            Crescendo:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('p'), staff[0])
            >>> abjad.attach(abjad.DynamicTrend('<'), staff[0])
            >>> abjad.attach(abjad.Dynamic('f'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    \p
                    \<
                    d'4
                    e'4
                    f'4
                    \f
                }

            Crescendo dal niente:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('niente', hide=True), staff[0])
            >>> abjad.attach(abjad.DynamicTrend('o<'), staff[0])
            >>> abjad.attach(abjad.Dynamic('f'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    - \tweak circled-tip ##t
                    \<
                    d'4
                    e'4
                    f'4
                    \f
                }

            Subito crescendo:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('p'), staff[0])
            >>> abjad.attach(abjad.DynamicTrend('<|'), staff[0])
            >>> abjad.attach(abjad.Dynamic('f'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    \p
                    - \tweak stencil #abjad-flared-hairpin
                    \<
                    d'4
                    e'4
                    f'4
                    \f
                }

            Subito crescendo dal niente:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('niente', hide=True), staff[0])
            >>> abjad.attach(abjad.DynamicTrend('o<|'), staff[0])
            >>> abjad.attach(abjad.Dynamic('f'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    - \tweak circled-tip ##t
                    - \tweak stencil #abjad-flared-hairpin
                    \<
                    d'4
                    e'4
                    f'4
                    \f
                }

        ..  container:: example

            Decrescendo:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('f'), staff[0])
            >>> abjad.attach(abjad.DynamicTrend('>'), staff[0])
            >>> abjad.attach(abjad.Dynamic('p'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    \f
                    \>
                    d'4
                    e'4
                    f'4
                    \p
                }

            Decrescendo al niente:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('f'), staff[0])
            >>> abjad.attach(abjad.DynamicTrend('>o'), staff[0])
            >>> abjad.attach(abjad.Dynamic('niente', command=r'\!'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    \f
                    - \tweak circled-tip ##t
                    \>
                    d'4
                    e'4
                    f'4
                    \!
                }

            Subito decrescendo:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('f'), staff[0])
            >>> abjad.attach(abjad.DynamicTrend('|>'), staff[0])
            >>> abjad.attach(abjad.Dynamic('p'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    \f
                    - \tweak stencil #abjad-flared-hairpin
                    \>
                    d'4
                    e'4
                    f'4
                    \p
                }

            Subito decrescendo al niente:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('f'), staff[0])
            >>> abjad.attach(abjad.DynamicTrend('|>o'), staff[0])
            >>> abjad.attach(abjad.Dynamic('niente', command=r'\!'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    \f
                    - \tweak circled-tip ##t
                    - \tweak stencil #abjad-flared-hairpin
                    \>
                    d'4
                    e'4
                    f'4
                    \!
                }

        ..  container:: example

            Constante:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('p'), staff[0])
            >>> abjad.attach(abjad.DynamicTrend('--'), staff[0])
            >>> abjad.attach(abjad.Dynamic('f'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    \p
                    - \tweak stencil #constante-hairpin
                    \<
                    d'4
                    e'4
                    f'4
                    \f
                }

        """
        return self._shape

    @property
    def spanner_start(self) -> bool:
        """
        Is true.

        ..  container:: example

            >>> abjad.DynamicTrend('<').spanner_start
            True

        """
        return True

    @property
    def tweaks(self) -> typing.Optional[LilyPondTweakManager]:
        r"""
        Gets tweaks

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('p'), staff[0])
            >>> start = abjad.DynamicTrend('<')
            >>> abjad.tweak(start).color = 'blue'
            >>> abjad.attach(start, staff[0])
            >>> abjad.attach(abjad.Dynamic('f'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    \p
                    - \tweak color #blue
                    \<
                    d'4
                    e'4
                    f'4
                    \f
                }

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Dynamic('p'), staff[0])
            >>> start = abjad.DynamicTrend(tweaks=[('color', 'blue')])
            >>> abjad.tweak(start).color = 'blue'
            >>> abjad.attach(start, staff[0])
            >>> abjad.attach(abjad.Dynamic('f'), staff[-1])
            >>> abjad.override(staff).dynamic_line_spanner.staff_padding = 4.5
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                \with
                {
                    \override DynamicLineSpanner.staff-padding = #4.5
                }
                {
                    c'4
                    \p
                    - \tweak color #blue
                    \<
                    d'4
                    e'4
                    f'4
                    \f
                }

        """
        return self._tweaks



import difflib
import os
import pathlib
from abjad.system.AbjadObject import AbjadObject


class TestManager(AbjadObject):
    """
    Manages test logic.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Managers'

    __slots__ = (
        )

    ### PRIVATE METHODS ###

    @staticmethod
    def _compare_backup(path):
        if isinstance(path, str):
            paths = [path]
        elif isinstance(path, pathlib.Path):
            paths = [str(path)]
        elif (isinstance(path, (tuple, list))):
            paths = [str(_) for _ in path]
        else:
            raise TypeError(path)
        for path in paths:
            backup_path = path + '.backup'
            if not TestManager.compare_files(path, backup_path):
                return False
        return True

    @staticmethod
    def _compare_lys(path_1, path_2):
        """
        Compares LilyPond file ``path_1`` to LilyPond file ``path_2``.

        Performs line-by-line comparison.

        Discards blank lines.

        Discards any LilyPond version statements.

        Discards any lines beginning with ``%``.

        Returns true or false.
        """
        file_1_lines = TestManager._normalize_ly(path_1)
        file_2_lines = TestManager._normalize_ly(path_2)
        return file_1_lines == file_2_lines

    @staticmethod
    def _compare_text_files(path_1, path_2):
        """
        Compares text file ``path_1`` to text file ``path_2``.

        Performs line-by-line comparison.

        Discards blank lines.

        Trims whitespace from the end of each line.

        Returns true or false.
        """
        file_1_lines, file_2_lines = [], []
        with open(path_1, 'r') as file_pointer:
            for line in file_pointer.readlines():
                line = line.strip()
                if line == '':
                    continue
                file_1_lines.append(line)
        with open(path_2, 'r') as file_pointer:
            for line in file_pointer.readlines():
                line = line.strip()
                if line == '':
                    continue
                file_2_lines.append(line)
        return file_1_lines == file_2_lines

    @staticmethod
    def _normalize_ly(path):
        lines = []
        with open(path, 'r') as file_pointer:
            for line in file_pointer.readlines():
                line = line.strip()
                if line == '':
                    continue
                if line.startswith(r'\version'):
                    continue
                elif line.startswith('%'):
                    continue
                lines.append(line)
        return lines

    ### PUBLIC METHODS ###

    @staticmethod
    def compare(string_1, string_2):
        """
        Compares ``string_1`` to ``string_2``.

        Massage newlines.

        Returns true or false.
        """
        if not isinstance(string_1, str):
            string_1 = format(string_1)
        split_lines = string_2.split('\n')
        if not split_lines[0] or split_lines[0].isspace():
            split_lines.pop(0)
        if not split_lines[-1] or split_lines[-1].isspace():
            split_lines.pop(-1)
        indent_width = 0
        if split_lines:
            for indent_width, character in enumerate(split_lines[0]):
                if character != ' ':
                    break
        massaged_lines = []
        for split_line in split_lines:
            massaged_line = split_line[indent_width:]
            massaged_lines.append(massaged_line)
        massaged_string = '\n'.join(massaged_lines)
        return string_1.replace('\t', '    ') == massaged_string

    @staticmethod
    def compare_files(path_1, path_2):
        """
        Compares file ``path_1`` to file ``path_2``.

        For all file types::

        * Performs line-by-line comparison
        * Discards blank lines

        For LilyPond files, additionally::

        * Discards any LilyPond version statements
        * Discards any lines beginning with ``%``

        Returns true when files compare the same and false when files compare
        differently.
        """
        path_1 = str(path_1)
        path_2 = str(path_2)
        if os.path.exists(path_1) and not os.path.exists(path_2):
            return False
        elif not os.path.exists(path_1) and os.path.exists(path_2):
            return False
        elif not os.path.exists(path_1) and not os.path.exists(path_2):
            return True
        if path_1.endswith('.backup'):
            path_1 = path_1.strip('.backup')
        if path_2.endswith('.backup'):
            path_2 = path_2.strip('.backup')
        base_1, extension_1 = os.path.splitext(path_1)
        base_2, extension_2 = os.path.splitext(path_2)
        assert extension_1 == extension_2
        if extension_1 == '.ly':
            return TestManager._compare_lys(path_1, path_2)
        else:
            return TestManager._compare_text_files(path_1, path_2)

    @staticmethod
    def compare_objects(object_one, object_two):
        """
        Compares ``object_one`` to ``object_two``.

        Returns true or false.
        """
        import abjad
        agent_one = abjad.StorageFormatManager(object_one)
        if agent_one.format_specification.coerce_for_equality:
            try:
                object_two = type(object_one)(object_two)
            except (
                TypeError,
                ValueError,
                ):
                return False
        elif not isinstance(object_two, type(object_one)):
            return False
        agent_two = abjad.StorageFormatManager(object_two)
        template_1 = agent_one.get_template_dict()
        template_2 = agent_two.get_template_dict()
        return template_1 == template_2

    @staticmethod
    def diff(object_a, object_b, title=None):
        """
        Gets diff of ``object_a`` and ``object_b`` formats.

        >>> one = abjad.Flute()

        >>> two = abjad.BassFlute()

        >>> diff = abjad.TestManager.diff(one, two, 'Diff:')
        >>> print(diff)
        Diff:
        - abjad.Flute(
        + abjad.BassFlute(
        ?       ++++
        -     name='flute',
        +     name='bass flute',
        ?           +++++
        -     short_name='fl.',
        +     short_name='bass fl.',
        ?                 +++++
              markup=abjad.Markup(
        -         contents=['Flute'],
        ?                    ^
        +         contents=['Bass flute'],
        ?                    ^^^^^^
                  ),
              short_markup=abjad.Markup(
        -         contents=['Fl.'],
        ?                    ^
        +         contents=['Bass fl.'],
        ?                    ^^^^^^
                  ),
              allowable_clefs=('treble',),
              context='Staff',
        -     middle_c_sounding_pitch=abjad.NamedPitch("c'"),
        ?                                              ^  -
        +     middle_c_sounding_pitch=abjad.NamedPitch('c'),
        ?                                              ^
        -     pitch_range=abjad.PitchRange('[C4, D7]'),
        ?                                     ^  ^^
        +     pitch_range=abjad.PitchRange('[C3, C6]'),
        ?                                     ^  ^^
        -     primary=True,
              )

        Returns string.
        """
        try:
            a_format = format(object_a, 'storage')
        except ValueError:
            a_format = format(object_a)
        try:
            b_format = format(object_b, 'storage')
        except ValueError:
            b_format = format(object_b)
        a_format = a_format.splitlines(True)
        b_format = b_format.splitlines(True)
        diff = ''.join(difflib.ndiff(a_format, b_format))
        if title is not None:
            diff = title + '\n' + diff
        return diff



from abjad.system.AbjadObject import AbjadObject


class WellformednessManager(AbjadObject):
    """
    Wellformedness manager.

    ..  container:: example

        >>> abjad.WellformednessManager()
        WellformednessManager()

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Managers'

    __slots__ = (
        '_allow_percussion_clef',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(self, allow_percussion_clef=None):
        self._allow_percussion_clef = allow_percussion_clef

    ### SPECIAL METHODS ###

    def __call__(self, argument=None):
        """
        Calls all wellformedness checks on ``argument``.

        Returns triples.
        """
        if argument is None:
            return
        check_names = [_ for _ in dir(self) if _.startswith('check_')]
        triples = []
        for current_check_name in sorted(check_names):
            current_check = getattr(self, current_check_name)
            current_violators, current_total = current_check(argument=argument)
            triple = (current_violators, current_total, current_check_name)
            triples.append(triple)
        return triples

    ### PRIVATE METHODS ###

    def _check_overlapping_spanners(self, argument, prototype=None):
        import abjad
        violators, spanners = set(), set()
        for leaf in abjad.iterate(argument).leaves():
            spanners_ = list(abjad.inspect(leaf).spanners(prototype))
            spanners.update(spanners_)
            if 1 < len(spanners_):
                if len(spanners_) == 2:
                    common_leaves = set(spanners_[0].leaves)
                    common_leaves &= set(spanners_[1].leaves)
                    if len(common_leaves) == 1:
                        leaf = list(common_leaves)[0]
                        if ((spanners_[0].leaves[0] is leaf and
                            spanners_[1].leaves[-1] is leaf) or
                            (spanners_[1].leaves[0] is leaf and
                            spanners_[0].leaves[-1] is leaf)):
                            break
                violators.update(spanners_)
        return violators, len(spanners)

    ### PUBLIC PROPERTIES ###

    @property
    def allow_percussion_clef(self):
        """
        Is true when manager allows percussion clef.

        Returns true, false or none.
        """
        return self._allow_percussion_clef

    ### PUBLIC METHODS ###

    def check_discontiguous_spanners(self, argument=None):
        """
        Checks discontiguous spanners.

        There are now two different types of spanner. Most spanners demand that
        spanner components be logical-voice-contiguous. But a few special
        spanners (like MetronomeMark) do not make such a demand. The check here
        consults the experimental `_contiguity_constraint`.

        Returns list of discontiguous spanners and nonnegative integer count of
        all spanners in ``argument``.
        """
        import abjad
        violators = []
        descendants = abjad.inspect(argument).descendants()
        spanners = abjad.inspect(descendants).spanners()
        for spanner in spanners:
            if spanner._contiguity_constraint == 'logical voice':
                if not spanner[:].are_contiguous_logical_voice():
                    violators.append(spanner)
        return violators, len(spanners)

    def check_duplicate_ids(self, argument=None):
        """
        Checks duplicate IDs.

        Returns violators and total.
        """
        import abjad
        violators = []
        components = abjad.iterate(argument).components()
        total_ids = [id(_) for _ in components]
        unique_ids = abjad.sequence(total_ids).remove_repeats()
        if len(unique_ids) < len(total_ids):
            for current_id in unique_ids:
                if 1 < total_ids.count(current_id):
                    violators.extend([_ for _ in components
                        if id(_) == current_id])
        return violators, len(total_ids)

    def check_empty_containers(self, argument=None):
        r"""
        Checks empty containers.

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> staff.append(abjad.Container())

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                f'4
                {
                }
            }

            >>> manager = abjad.WellformednessManager()
            >>> violators, total = manager.check_empty_containers(staff)
            >>> violators
            [Container()]

        Returns list of empty containers and count of all containers in
        ``argument``.
        """
        import abjad
        violators, containers = [], set()
        for container in abjad.iterate(argument).components(abjad.Container):
            containers.add(container)
            if len(container) == 0:
                violators.append(container)
        return violators, len(containers)

    def check_misdurated_measures(self, argument=None):
        """
        Checks misdurated measures.

        Returns violators and total.
        """
        import abjad
        violators, total = [], set()
        for measure in abjad.iterate(argument).components(abjad.Measure):
            total.add(measure)
            time_signature = measure.time_signature
            if time_signature is not None:
                duration = measure._get_preprolated_duration()
                if duration != time_signature.duration:
                    violators.append(measure)
        return violators, len(total)

    def check_misfilled_measures(self, argument=None):
        """
        Checks misfilled measures.

        Returns violators and total.
        """
        import abjad
        violators, total = [], set()
        for measure in abjad.iterate(argument).components(abjad.Measure):
            total.add(measure)
            if measure.is_misfilled:
                violators.append(measure)
        return violators, len(total)

    def check_mismatched_enchained_hairpins(self, argument=None):
        r"""
        Checks mismatched enchained hairpins.

        ..  container:: example

            Checks mismatched enchained hairpins:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Hairpin('p < f'), staff[:2])
            >>> abjad.attach(abjad.Hairpin('p > pp'), staff[1:])

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness())
            0 /	2 discontiguous spanners
            0 /	5 duplicate ids
            0 / 1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            2 /	2 mismatched enchained hairpins
            0 / 0 mispitched ties
            0 /	4 misrepresented flags
            0 /	5 missing parents
            0 /	0 nested measures
            0 / 4 notes on wrong clef
            0 / 4 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            0 /	2 overlapping hairpins
            0 /	0 overlapping octavation spanners
            0 /	0 overlapping ties
            0 / 0 overlapping trill spanners 
            0 /	0 tied rests

        Enchained hairpins are fine so long as hairpin ends match.

        Returns violators and total.
        """
        import abjad
        violators, total = [], set()
        for leaf in abjad.iterate(argument).leaves():
            hairpins = abjad.inspect(leaf).spanners(abjad.Hairpin)
            hairpins = list(hairpins)
            total.update(hairpins)
            if len(hairpins) <= 1:
                continue
            if 2 < len(hairpins):
                raise Exception('too many hairpins')
            assert len(hairpins) == 2
            hairpins_are_enchained = False
            if leaf is hairpins[0][-1] and leaf is hairpins[-1][0]:
                hairpins_are_enchained = True
            if leaf is hairpins[-1][-1] and leaf is hairpins[0][0]:
                hairpins_are_enchained = True
            if not hairpins_are_enchained:
                continue
            if leaf is hairpins[0][0]:
                first_hairpin = hairpins[-1]
                second_hairpin = hairpins[0]
            else:
                first_hairpin = hairpins[0]
                second_hairpin = hairpins[-1]
            if first_hairpin.stop_dynamic == second_hairpin.start_dynamic:
                continue
            else:
                violators.append(first_hairpin)
                violators.append(second_hairpin)
        return violators, len(total)

    def check_mispitched_ties(self, argument=None):
        r"""
        Checks mispitched notes.

        ..  container:: example

            Checks for mispitched ties attached to notes:

            >>> staff = abjad.Staff("c'4 ~ c'")
            >>> staff[1].written_pitch = abjad.NamedPitch("d'")

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness())
            0 /	1 discontiguous spanners
            0 /	3 duplicate ids
            0 / 1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            0 /	0 mismatched enchained hairpins
            1 /	1 mispitched ties
            0 /	2 misrepresented flags
            0 /	3 missing parents
            0 /	0 nested measures
            0 / 2 notes on wrong clef
            0 / 2 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            0 /	0 overlapping hairpins
            0 /	0 overlapping octavation spanners
            0 /	1 overlapping ties
            0 / 0 overlapping trill spanners
            0 /	0 tied rests

        ..  container:: example

            Checks for mispitched ties attached to chords:

            >>> staff = abjad.Staff("<c' d' bf'>4 ~ <c' d' bf'>")
            >>> staff[1].written_pitches = [6, 9]

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness())
            0 /	1 discontiguous spanners
            0 /	3 duplicate ids
            0 / 1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            0 /	0 mismatched enchained hairpins
            1 /	1 mispitched ties
            0 /	2 misrepresented flags
            0 /	3 missing parents
            0 /	0 nested measures
            0 / 2 notes on wrong clef
            0 / 2 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            0 /	0 overlapping hairpins
            0 /	0 overlapping octavation spanners
            0 /	1 overlapping ties
            0 / 0 overlapping trill spanners
            0 /	0 tied rests

        Returns violator ties together with total number of ties.
        """
        import abjad
        violators, ties = [], set()
        for leaf in abjad.iterate(argument).leaves(pitched=True):
            ties_ = abjad.inspect(leaf).spanners(abjad.Tie)
            ties.update(ties_)
        for tie in ties:
            for first_leaf, second_leaf in abjad.sequence(tie).nwise():
                first_pitches = abjad.inspect(first_leaf).pitches()
                first_pitches = set([_.number for _ in first_pitches])
                second_pitches = abjad.inspect(second_leaf).pitches()
                second_pitches = set([_.number for _ in second_pitches])
                if not (first_pitches & second_pitches):
                    violators.append(tie)
                    break
        return violators, len(ties)

    def check_misrepresented_flags(self, argument=None):
        """
        Checks misrepresented flags.

        Returns violators and total.
        """
        import abjad
        violators, total = [], set()
        for leaf in abjad.iterate(argument).leaves():
            total.add(leaf)
            flags = leaf.written_duration.flag_count
            left = getattr(abjad.setting(leaf), 'stem_left_beam_count', None)
            right = getattr(abjad.setting(leaf), 'stem_right_beam_count', None)
            if left is not None:
                if (flags < left or
                    (left < flags and right not in (flags, None))):
                    if leaf not in violators:
                        violators.append(leaf)
            if right is not None:
                if (flags < right or
                    (right < flags and left not in (flags, None))):
                    if leaf not in violators:
                        violators.append(leaf)
        return violators, len(total)

    def check_missing_parents(self, argument=None):
        """
        Checks missing parents.

        Returns violators and total.
        """
        import abjad
        violators, total = [], set()
        components = abjad.iterate(argument).components()
        for i, component in enumerate(components):
            total.add(component)
            if 0 < i:
                if abjad.inspect(component).parentage().parent is None:
                    violators.append(component)
        return violators, len(total)

    def check_nested_measures(self, argument=None):
        """
        Checks nested measures.

        Returns violators and total.
        """
        import abjad
        violators, total = [], set()
        for measure in abjad.iterate(argument).components(abjad.Measure):
            total.add(measure)
            parentage = abjad.inspect(measure).parentage(
                include_self=False,
                )
            if parentage.get_first(abjad.Measure):
                violators.append(measure)
        return violators, len(total)

    def check_notes_on_wrong_clef(self, argument=None):
        r"""
        Checks notes and chords on wrong clef.

        ..  container:: example

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> clef = abjad.Clef('alto')
            >>> abjad.attach(clef, staff[0])
            >>> violin = abjad.Violin()
            >>> abjad.attach(violin, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \clef "alto"
                    c'8
                    d'8
                    e'8
                    f'8
                }

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness())
            0 /	0 discontiguous spanners
            0 /	5 duplicate ids
            0 /	1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            0 /	0 mismatched enchained hairpins
            0 / 0 mispitched ties
            0 /	4 misrepresented flags
            0 /	5 missing parents
            0 /	0 nested measures
            4 /	4 notes on wrong clef
            0 /	4 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            0 /	0 overlapping hairpins
            0 /	0 overlapping octavation spanners
            0 /	0 overlapping ties
            0 / 0 overlapping trill spanners
            0 /	0 tied rests

        ..  container:: example

            Allows percussion clef:

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
            >>> clef = abjad.Clef('percussion')
            >>> abjad.attach(clef, staff[0])
            >>> violin = abjad.Violin()
            >>> abjad.attach(violin, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    \clef "percussion"
                    c'8
                    d'8
                    e'8
                    f'8
                }

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness(
            ...     allow_percussion_clef=True,
            ...     ))
            0 /	0 discontiguous spanners
            0 /	5 duplicate ids
            0 /	1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            0 /	0 mismatched enchained hairpins
            0 / 0 mispitched ties
            0 /	4 misrepresented flags
            0 /	5 missing parents
            0 /	0 nested measures
            0 /	4 notes on wrong clef
            0 /	4 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            0 /	0 overlapping hairpins
            0 /	0 overlapping octavation spanners
            0 /	0 overlapping ties
            0 / 0 overlapping trill spanners
            0 /	0 tied rests

        ..  container:: example

            Forbids percussion clef:

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness(
            ...     allow_percussion_clef=False,
            ...     ))
            0 /	0 discontiguous spanners
            0 /	5 duplicate ids
            0 /	1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            0 /	0 mismatched enchained hairpins
            0 / 0 mispitched ties
            0 /	4 misrepresented flags
            0 /	5 missing parents
            0 /	0 nested measures
            4 /	4 notes on wrong clef
            0 /	4 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            0 /	0 overlapping hairpins
            0 /	0 overlapping octavation spanners
            0 /	0 overlapping ties
            0 / 0 overlapping trill spanners
            0 /	0 tied rests

        Returns true or false.
        """
        import abjad
        violators, total = [], set()
        for leaf in abjad.iterate(argument).leaves():
            total.add(leaf)
            instrument = abjad.inspect(leaf).effective(abjad.Instrument)
            if instrument is None:
                continue
            clef = abjad.inspect(leaf).effective(abjad.Clef)
            if clef is None:
                continue
            allowable_clefs = [
                abjad.Clef(_) for _ in instrument.allowable_clefs
                ]
            if self.allow_percussion_clef:
                allowable_clefs.append(abjad.Clef('percussion'))
            if clef not in allowable_clefs:
                violators.append(leaf)
        return violators, len(total)

    def check_out_of_range_notes(self, argument=None):
        r"""
        Checks out-of-range notes.

        ..  container:: example

            Out of range:

            >>> staff = abjad.Staff("c'8 r8 <d fs>8 r8")
            >>> violin = abjad.Violin()
            >>> abjad.attach(violin, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'8
                    r8
                    <d fs>8
                    r8
                }

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness())
            0 /	0 discontiguous spanners
            0 /	5 duplicate ids
            0 /	1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            0 /	0 mismatched enchained hairpins
            0 / 0 mispitched ties
            0 /	4 misrepresented flags
            0 /	5 missing parents
            0 /	0 nested measures
            0 /	4 notes on wrong clef
            1 /	2 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            0 /	0 overlapping hairpins
            0 /	0 overlapping octavation spanners
            0 /	0 overlapping ties
            0 / 0 overlapping trill spanners
            0 /	2 tied rests

        Returns true or false.
        """
        import abjad
        violators, total = [], set()
        for leaf in abjad.iterate(argument).leaves(pitched=True):
            total.add(leaf)
            instrument = abjad.inspect(leaf).effective(abjad.Instrument)
            if instrument is None:
                continue
            if leaf not in instrument.pitch_range:
                violators.append(leaf)
        return violators, len(total)

    def check_overlapping_beams(self, argument=None):
        r"""
        Checks overlapping beams.

        ..  container:: example

            Overlapping beams are well-formed:

            >>> staff = abjad.Staff("c'8 d' e' f'")
            >>> abjad.attach(abjad.Beam(), staff[:3])
            >>> abjad.attach(abjad.Beam(), staff[2:])

            >>> abjad.f(staff)
            \new Staff
            {
                c'8
                [
                d'8
                e'8
                ]
                [
                f'8
                ]
            }

            >>> manager = abjad.WellformednessManager()
            >>> violators, total = manager.check_overlapping_beams(staff)
            >>> for beam in sorted(violators):
            ...     beam
            ...
            Beam("c'8, d'8, e'8", durations=(), span_beam_count=1)
            Beam("e'8, f'8", durations=(), span_beam_count=1)

            (Enchained beams are not well-formed either.)

        Returns list of overlapping beams and nonnegative integer count of
        total beams in score.
        """
        import abjad
        violators, total = [], set()
        for leaf in abjad.iterate(argument).leaves():
            beams = abjad.inspect(leaf).spanners(abjad.Beam)
            total.update(beams)
            if 1 < len(beams):
                for beam in beams:
                    if beam not in violators:
                        violators.append(beam)
        return violators, len(total)

    def check_overlapping_glissandi(self, argument=None):
        """
        Checks overlapping glissandi.

        Returns violators and total.
        """
        import abjad
        return self._check_overlapping_spanners(argument, abjad.Glissando)

    def check_overlapping_hairpins(self, argument=None):
        r"""
        Checks overlapping hairpins.

        ..  container:: example

            Checks overlapping hairpins:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.Hairpin('<'), staff[:])
            >>> abjad.attach(abjad.Hairpin('>'), staff[:])

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness())
            0 /	2 discontiguous spanners
            0 /	5 duplicate ids
            0 / 1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            0 /	2 mismatched enchained hairpins
            0 / 0 mispitched ties
            0 /	4 misrepresented flags
            0 /	5 missing parents
            0 /	0 nested measures
            0 /	4 notes on wrong clef
            0 /	4 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            2 /	2 overlapping hairpins
            0 /	0 overlapping octavation spanners
            0 /	0 overlapping ties
            0 / 0 overlapping trill spanners
            0 /	0 tied rests

        Enchained hairpins are fine so long as hairpin ends match.

        Returns violators and total.
        """
        import abjad
        return self._check_overlapping_spanners(argument, abjad.Hairpin)

    def check_overlapping_octavation_spanners(self, argument=None):
        """
        Checks overlapping octavation spanners.

        Returns violators and total.
        """
        import abjad
        violators, total = [], set()
        prototype = abjad.OctavationSpanner
        for leaf in abjad.iterate(argument).leaves():
            spanners = abjad.inspect(leaf).spanners(prototype)
            total.update(spanners)
            if 1 < len(spanners):
                for spanner in spanners:
                    if spanner not in violators:
                        violators.append(spanner)
        return violators, len(total)

    def check_overlapping_ties(self, argument=None):
        r"""
        Checks overlapping ties.

        ..  container:: example

            Checks overlapping ties:

            >>> staff = abjad.Staff("c'4 c' c' c''")
            >>> abjad.attach(abjad.Tie(), staff[:2])
            >>> tie = abjad.Tie()
            >>> tie._ignore_attachment_test = True
            >>> tie._ignore_before_attach = True
            >>> abjad.attach(tie, staff[1:3])

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness())
            0 /	2 discontiguous spanners
            0 /	5 duplicate ids
            0 / 1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            0 /	0 mismatched enchained hairpins
            0 / 2 mispitched ties
            0 /	4 misrepresented flags
            0 /	5 missing parents
            0 /	0 nested measures
            0 /	4 notes on wrong clef
            0 /	4 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            0 /	0 overlapping hairpins
            0 /	0 overlapping octavation spanners
            2 /	2 overlapping ties
            0 / 0 overlapping trill spanners
            0 /	0 tied rests

        Returns violators and count of total ties.
        """
        import abjad
        violators, total = [], set()
        for leaf in abjad.iterate(argument).leaves():
            spanners = abjad.inspect(leaf).spanners(abjad.Tie)
            total.update(spanners)
            if 1 < len(spanners):
                for spanner in spanners:
                    if spanner not in violators:
                        violators.append(spanner)
        return violators, len(total)

    def check_overlapping_trill_spanners(self, argument=None):
        r"""
        Checks overlapping trill spanners.

        ..  container:: example

            Enchained trill spanners are ok:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.TrillSpanner(), staff[:3])
            >>> abjad.attach(abjad.TrillSpanner(), staff[2:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    \startTrillSpan
                    d'4
                    e'4
                    \stopTrillSpan
                    \startTrillSpan
                    f'4
                    \stopTrillSpan
                }

            >>> abjad.inspect(staff).is_well_formed()
            True

        ..  container:: example

            Overlapping trill spanners are not well-formed:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.attach(abjad.TrillSpanner(), staff[:])
            >>> abjad.attach(abjad.TrillSpanner(), staff[:])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    \startTrillSpan
                    \startTrillSpan
                    d'4
                    e'4
                    f'4
                    \stopTrillSpan
                    \stopTrillSpan
                }

            >>> agent = abjad.inspect(staff)
            >>> print(agent.tabulate_wellformedness())
            0 /	2 discontiguous spanners
            0 /	5 duplicate ids
            0 /	1 empty containers
            0 /	0 misdurated measures
            0 /	0 misfilled measures
            0 /	0 mismatched enchained hairpins
            0 / 0 mispitched ties
            0 /	4 misrepresented flags
            0 /	5 missing parents
            0 /	0 nested measures
            0 /	4 notes on wrong clef
            0 /	4 out of range notes
            0 /	0 overlapping beams
            0 /	0 overlapping glissandi
            0 /	0 overlapping hairpins
            0 /	0 overlapping octavation spanners
            0 /	0 overlapping ties
            2 /	2 overlapping trill spanners
            0 /	0 tied rests

        Enchained hairpins are fine so long as hairpin ends match.

        Returns violators and total.
        """
        import abjad
        return self._check_overlapping_spanners(argument, abjad.TrillSpanner)

    def check_tied_rests(self, argument=None):
        """
        Checks tied rests.

        Returns violators and total.
        """
        import abjad
        violators, total = [], set()
        for rest in abjad.iterate(argument).leaves(abjad.Rest):
            total.add(rest)
            if abjad.inspect(rest).has_spanner(abjad.Tie):
                violators.append(rest)
        return violators, len(total)



import math


def integer_to_base_k_tuple(n, k):
    """
    Changes nonnegative integer ``n`` to base-`k` tuple.

    ..  container:: example

        Gets base-10 digits of 1066:

        >>> abjad.mathtools.integer_to_base_k_tuple(1066, 10)
        (1, 0, 6, 6)

    ..  container:: example

        Gets base-2 digits of 1066:

        >>> abjad.mathtools.integer_to_base_k_tuple(1066, 2)
        (1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0)

    ..  container:: example

        Gets base-26 digits of 1066:

        >>> abjad.mathtools.integer_to_base_k_tuple(1066, 26)
        (1, 15, 0)

    Returns tuple of positive integers.
    """
    assert isinstance(n, int), repr(n)
    assert 0 <= n, repr(n)
    if n == 0:
        return (0,)
    result = []
    current_exponent = math.trunc(math.log(n, k))
    remainder = n
    while 0 <= current_exponent:
        current_power = k ** current_exponent
        current_digit = remainder // current_power
        result.append(current_digit)
        remainder -= current_digit * current_power
        current_exponent -= 1
    return tuple(result)



from abjad import mathtools
from abjad.system.FormatSpecification import FormatSpecification
from . import constants
from .Pitch import Pitch


class NumberedPitch(Pitch):
    r"""
    Numbered pitch.

    ..  container:: example

        Initializes from number:

        >>> numbered_pitch = abjad.NumberedPitch(13)
        >>> abjad.show(numbered_pitch) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(numbered_pitch.__illustrate__()[abjad.Staff])
            \new Staff
            \with
            {
                \override TimeSignature.stencil = ##f
            }
            {
                \clef "treble"
                cs''1 * 1/4
            }

    ..  container:: example

        Initializes from other numbered pitch

        >>> numbered_pitch = abjad.NumberedPitch(abjad.NumberedPitch(13))
        >>> abjad.show(numbered_pitch) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(numbered_pitch.__illustrate__()[abjad.Staff])
            \new Staff
            \with
            {
                \override TimeSignature.stencil = ##f
            }
            {
                \clef "treble"
                cs''1 * 1/4
            }

    ..  container:: example

        Initializes from pitch-class / octave pair:

        >>> numbered_pitch = abjad.NumberedPitch((1, 5))
        >>> abjad.show(numbered_pitch) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(numbered_pitch.__illustrate__()[abjad.Staff])
            \new Staff
            \with
            {
                \override TimeSignature.stencil = ##f
            }
            {
                \clef "treble"
                cs''1 * 1/4
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_number',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        number=0, *,
        arrow=None,
        octave=None,
    ):
        super().__init__(
            number or 0,
            arrow=arrow,
            octave=octave,
            )

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        """
        Adds `argument` to numbered pitch.

        ..  container:: example

            >>> abjad.NumberedPitch(12) + abjad.NumberedPitch(13)
            NumberedPitch(25)

            >>> abjad.NumberedPitch(13) + abjad.NumberedPitch(12)
            NumberedPitch(25)

        Returns new numbered pitch.
        """
        argument = type(self)(argument)
        semitones = float(self) + float(argument)
        return type(self)(semitones)

    def __lt__(self, argument):
        r"""Is true when `argument` can be coerced to a numbered pitch and when this
        numbered pitch is less than `argument`.

        ..  container:: example

            >>> pitch_1 = abjad.NumberedPitch(12)
            >>> pitch_2 = abjad.NumberedPitch(12)
            >>> pitch_3 = abjad.NumberedPitch(13)

            >>> pitch_1 < pitch_1
            False
            >>> pitch_1 < pitch_2
            False
            >>> pitch_1 < pitch_3
            True

            >>> pitch_2 < pitch_1
            False
            >>> pitch_2 < pitch_2
            False
            >>> pitch_2 < pitch_3
            True

            >>> pitch_3 < pitch_1
            False
            >>> pitch_3 < pitch_2
            False
            >>> pitch_3 < pitch_3
            False

        Returns true or false.
        """
        try:
            argument = type(self)(argument)
        except (ValueError, TypeError):
            return False
        return self.number < argument.number

    def __neg__(self):
        """
        Negates numbered pitch.

        ..  container:: example

            >>> -abjad.NumberedPitch(13.5)
            NumberedPitch(-13.5)

            >>> -abjad.NumberedPitch(-13.5)
            NumberedPitch(13.5)

        Returns new numbered pitch.
        """
        return type(self)(-self.number)

    def __radd__(self, argument):
        """
        Adds numbered pitch to `argument`.

        ..  container:: example

            >>> pitch = abjad.NumberedPitch(13)
            >>> abjad.NumberedPitch(12).__radd__(pitch)
            NumberedPitch(25)

            >>> pitch = abjad.NumberedPitch(12)
            >>> abjad.NumberedPitch(13).__radd__(pitch)
            NumberedPitch(25)

        Returns new numbered pitch.
        """
        argument = type(self)(argument)
        return argument.__add__(self)

    def __str__(self):
        """
        Gets string representation of numbered pitch.

        Returns string.
        """
        return str(self.number)

    def __sub__(self, argument):
        """
        Subtracts `argument` from numbered pitch.

        ..  container:: example

            >>> abjad.NumberedPitch(12) - abjad.NumberedPitch(12)
            NumberedInterval(0)

            >>> abjad.NumberedPitch(12) - abjad.NumberedPitch(13)
            NumberedInterval(1)

            >>> abjad.NumberedPitch(13) - abjad.NumberedPitch(12)
            NumberedInterval(-1)

        Returns numbered interval.
        """
        import abjad
        if isinstance(argument, type(self)):
            return abjad.NumberedInterval.from_pitch_carriers(
                self, argument)
        interval = abjad.NumberedInterval(argument)
        interval = -interval
        return interval.transpose(self)

    ### PRIVATE METHODS ###

    def _apply_accidental(self, accidental=None):
        import abjad
        accidental = abjad.Accidental(accidental)
        semitones = self.number + accidental.semitones
        return type(self)(semitones)

    def _from_named_parts(self, dpc_number, alteration, octave):
        import abjad
        pc_number = constants._diatonic_pc_number_to_pitch_class_number[dpc_number]
        pc_number += alteration
        pc_number += (octave - 4) * 12
        self._number = mathtools.integer_equivalent_number_to_integer(pc_number)
        octave_number, pc_number = divmod(self._number, 12)
        self._pitch_class = abjad.NumberedPitchClass(pc_number)
        self._octave = abjad.Octave(octave_number + 4)

    def _from_number(self, number):
        import abjad
        self._number = self._to_nearest_quarter_tone(number)
        octave_number, pc_number = divmod(self._number, 12)
        self._octave = abjad.Octave(octave_number + 4)
        self._pitch_class = abjad.NumberedPitchClass(pc_number)

    def _from_pitch_or_pitch_class(self, pitch_or_pitch_class):
        import abjad
        self._number = self._to_nearest_quarter_tone(float(pitch_or_pitch_class))
        octave_number, pc_number = divmod(self._number, 12)
        self._octave = abjad.Octave(octave_number + 4)
        self._pitch_class = abjad.NumberedPitchClass(
            pc_number,
            arrow=pitch_or_pitch_class.arrow,
            )

    def _get_diatonic_pc_name(self):
        return self.pitch_class._get_diatonic_pc_name()

    def _get_diatonic_pc_number(self):
        return self.numbered_pitch_class._get_diatonic_pc_number()

    def _get_diatonic_pitch_number(self):
        result = 7 * (self.octave.number - 4)
        result += self._get_diatonic_pc_number()
        return result

    def _get_format_specification(self):
        return FormatSpecification(
            client=self,
            coerce_for_equality=True,
            repr_is_indented=False,
            storage_format_is_indented=False,
            storage_format_args_values=[self.number],
            storage_format_kwargs_names=['arrow'],
            )

    def _get_lilypond_format(self):
        return self.name

    ### PUBLIC PROPERTIES ###

    @property
    def accidental(self):
        """
        Gets accidental of numbered pitch.

        ..  container:: example

            >>> abjad.NumberedPitchClass(13).accidental
            Accidental('sharp')

        Returns accidental.
        """
        return self.pitch_class.accidental

    @property
    def arrow(self):
        """
        Gets arrow of numbered pitch.

        ..  container:: example

            Gets no arrow:

            >>> abjad.NumberedPitch(13).arrow is None
            True

        ..  container:: example

            Gets up-arrow:

            >>> abjad.NumberedPitch(13, arrow=abjad.Up).arrow
            Up

        ..  container:: example

            Gets down-arrow:

            >>> abjad.NumberedPitch(13, arrow=abjad.Down).arrow
            Down

        Returns up, down or none.
        """
        return self._pitch_class.arrow

    @property
    def hertz(self):
        """
        Gets frequency of numbered pitch in Hertz.

        ..  container:: example

            >>> abjad.NumberedPitch(9).hertz
            440.0

            >>> abjad.NumberedPitch(0).hertz
            261.62...

            >>> abjad.NumberedPitch(12).hertz
            523.25...

        Returns float.
        """
        return super().hertz

    @property
    def name(self):
        """
        Gets name of numbered pitch.

        ..  container:: example

            >>> abjad.NumberedPitch(13).name
            "cs''"

        Returns string
        """
        return '{}{}'.format(
            self.pitch_class.name,
            self.octave.ticks,
            )

    @property
    def number(self):
        """
        Gets number of numbered pitch.

        ..  container:: example

            >>> abjad.NumberedPitch(13).number
            13

        Returns number.
        """
        pc_number = float(self.pitch_class)
        octave_base_pitch = (self.octave.number - 4) * 12
        return mathtools.integer_equivalent_number_to_integer(
            pc_number + octave_base_pitch
            )

    @property
    def octave(self):
        """
        Gets octave of numbered pitch.

        ..  container:: example

            >>> abjad.NumberedPitch(13).octave
            Octave(5)

        Returns octave.
        """
        return self._octave

    @property
    def pitch_class(self):
        """
        Gets pitch-class of numbered pitch.

        ..  container:: example

            >>> abjad.NumberedPitch(13).pitch_class
            NumberedPitchClass(1)

        Returns numbered pitch-class.
        """
        return self._pitch_class

    ### PUBLIC METHODS ###

    @classmethod
    def from_hertz(class_, hertz):
        """
        Makes numbered pitch from `hertz`.

        ..  container:: example

            >>> abjad.NumberedPitch.from_hertz(440)
            NumberedPitch(9)

        ..  container:: example

            REGRESSION. Returns 12 (not 0):

            >>> abjad.NumberedPitch.from_hertz(519)
            NumberedPitch(12)

        Returns newly constructed numbered pitch.
        """
        return super().from_hertz(hertz)

    def get_name(self, locale=None):
        """
        Gets name of numbered pitch name according to `locale`.

        ..  container:: example

            >>> abjad.NumberedPitch(13).get_name()
            "cs''"

            >>> abjad.NumberedPitch(13).get_name(locale='us')
            'C#5'

        Set `locale` to `'us'` or none.

        Returns string.
        """
        import abjad
        return abjad.NamedPitch(self).get_name(locale=locale)

    def interpolate(self, stop_pitch, fraction):
        """
        Interpolates between numbered pitch and `stop_pitch` by `fraction`.

        ..  container:: example

            Interpolates from C4 to C5:

            >>> start_pitch = abjad.NumberedPitch(0)
            >>> stop_pitch = abjad.NumberedPitch(12)

            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(0))
            NumberedPitch(0)
            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(1, 4))
            NumberedPitch(3)
            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(1, 2))
            NumberedPitch(6)
            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(3, 4))
            NumberedPitch(9)
            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(1))
            NumberedPitch(12)

        ..  container:: example

            Interpolates from C5 to C4:

            >>> start_pitch = abjad.NumberedPitch(12)
            >>> stop_pitch = abjad.NumberedPitch(0)

            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(0))
            NumberedPitch(12)
            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(1, 4))
            NumberedPitch(9)
            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(1, 2))
            NumberedPitch(6)
            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(3, 4))
            NumberedPitch(3)
            >>> start_pitch.interpolate(stop_pitch, abjad.Fraction(1))
            NumberedPitch(0)

        Returns new numbered pitch.
        """
        import abjad
        assert 0 <= fraction <= 1, repr(fraction)
        stop_pitch = type(self)(stop_pitch)
        distance = stop_pitch - self
        distance = abs(distance.semitones)
        distance = fraction * distance
        distance = int(distance)
        if stop_pitch < self:
            distance *= -1
        pitch_number = self.number
        pitch_number = pitch_number + distance
        pitch = abjad.NumberedPitch(pitch_number)
        if self <= stop_pitch:
            triple = (self, pitch, stop_pitch)
            assert self <= pitch <= stop_pitch, triple
        else:
            triple = (self, pitch, stop_pitch)
            assert self >= pitch >= stop_pitch, triple
        return pitch

    def invert(self, axis=None):
        """
        Inverts numbered pitch around `axis`.

        ..  container:: example

            Inverts pitch-class about pitch-class 0 explicitly:

            >>> abjad.NumberedPitch(2).invert(0)
            NumberedPitch(-2)

            >>> abjad.NumberedPitch(-2).invert(0)
            NumberedPitch(2)

        ..  container:: example

            Inverts pitch-class about pitch-class 0 implicitly:

            >>> abjad.NumberedPitch(2).invert()
            NumberedPitch(-2)

            >>> abjad.NumberedPitch(-2).invert()
            NumberedPitch(2)

        ..  container:: example

            Inverts pitch-class about pitch-class -3:

            >>> abjad.NumberedPitch(2).invert(-3)
            NumberedPitch(-8)

        Returns new numbered pitch.
        """
        return Pitch.invert(self, axis=axis)

    def multiply(self, n=1):
        """
        Multiplies numbered pitch by index `n`.

        ..  container:: example

            >>> abjad.NumberedPitch(14).multiply(3)
            NumberedPitch(42)

        Returns new numbered pitch.
        """
        return super().multiply(n=n)

    def transpose(self, n=0):
        """
        Tranposes numbered pitch by `n` semitones.

        ..  container:: example

            >>> abjad.NumberedPitch(13).transpose(1)
            NumberedPitch(14)

        Returns new numbered pitch.
        """
        import abjad
        interval = abjad.NumberedInterval(n)
        return type(self)(float(self) + float(interval))



from abjad.system.AbjadObject import AbjadObject


class LilyPondVersionToken(AbjadObject):
    r"""
    A LilyPond file ``\version`` token.

    ..  container:: example

        >>> abjad.LilyPondVersionToken() # doctest: +SKIP
        LilyPondVersionToken('2.19.0')

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_version_string',
        )

    ### INITIALIZER ###

    def __init__(self, version_string=None):
        from abjad import abjad_configuration
        assert isinstance(version_string, (str, type(None)))
        if version_string is None:
            version_string = abjad_configuration.get_lilypond_version_string()
        self._version_string = version_string

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r"""
        Formats LilyPond version token.

        ..  container:: example

            >>> token = abjad.LilyPondVersionToken()
            >>> print(format(token)) # doctest: +SKIP
            \version "2.19.0"

        Return string.
        """
        import abjad
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return abjad.StorageFormatManager(self).get_storage_format()
        return str(self)

    def __repr__(self):
        """
        Gets interpreter representation of LilyPond version_string token.

        ..  container:: example

            >>> token = abjad.LilyPondVersionToken()
            >>> token # doctest: +SKIP
            LilyPondVersionToken('2.19.0')

        Returns string.
        """
        return '{}({!r})'.format(type(self).__name__, self.version_string)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return r'\version "{}"'.format(self.version_string)

    ### PUBLIC PROPERTIES ###

    @property
    def version_string(self):
        """
        Gets version string of LilyPond version token.

        ..  container:: example

            Gets version string from install environment:

            >>> token = abjad.LilyPondVersionToken(
            ...     version_string=None,
            ...     )
            >>> token.version_string # doctest: +SKIP
            '2.19.0'

        ..  container:: example

            Gets version string from explicit input:

            >>> token = abjad.LilyPondVersionToken(
            ...     version_string='2.19.0',
            ...     )
            >>> token.version_string
            '2.19.0'

        Returns string.
        """
        return self._version_string



import abjad


class {class_name}(abjad.AbjadObject):
    """
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    def __call__(self):
        pass

    ### PUBLIC PROPERTIES ###
